# -*- coding: utf-8 -*-
#
# Integeration tests specific to the mailer
#

from collective.easyform.api import get_actions
from collective.easyform.api import get_schema
from collective.easyform.api import set_actions
from collective.easyform.api import set_fields
from collective.easyform.interfaces import IActionExtender
from collective.easyform.tests import base
from plone import api
from plone.app.textfield.value import RichTextValue
from plone.namedfile.file import NamedFile

import email


class TestFunctions(base.EasyFormTestCase):
    """ Test mailer action """

    def dummy_send(self, mfrom, mto, messageText, immediate=False):
        self.mfrom = mfrom
        self.mto = mto
        self.messageText = messageText
        self.messageBody = '\n\n'.join(messageText.split('\n\n')[1:])

    def afterSetUp(self):
        super(TestFunctions, self).afterSetUp()
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.CSRFProtection = False  # no csrf protection
        self.mailhost = self.folder.MailHost
        self.mailhost._send = self.dummy_send
        actions = get_actions(self.ff1)
        actions['mailer'].recipient_email = u'mdummy@address.com'
        set_actions(self.ff1, actions)

    def LoadRequestForm(self, **kwargs):
        request = self.layer['request']
        request.form.clear()
        prefix = 'form.widgets.'
        for key in kwargs.keys():
            request.form[prefix + key] = kwargs[key]
        return request

    def test_DummyMailer(self):
        """ sanity check; make sure dummy mailer works as expected """

        self.mailhost.send(
            'messageText', mto='dummy@address.com', mfrom='dummy1@address.com')
        self.assertTrue(self.messageText.endswith('messageText'))
        self.assertEqual(self.mto, ['dummy@address.com'])
        self.assertIn('To: dummy@address.com', self.messageText)
        self.assertEqual(self.mfrom, 'dummy1@address.com')
        self.assertIn('From: dummy1@address.com', self.messageText)

    def test_Mailer_Basic(self):
        """ Test mailer with dummy_send """

        mailer = get_actions(self.ff1)['mailer']

        data = {'topic': 'test subject', 'comments': 'test comments'}
        request = self.LoadRequestForm(**data)

        mailer.onSuccess(data, request)

        self.assertIn('To: mdummy@address.com', self.messageText)
        self.assertIn(
            'Subject: =?utf-8?q?test_subject?=', self.messageText)
        msg = email.message_from_string(self.messageText)
        self.assertIn(
            'test comments', msg.get_payload(decode=True))

    def test_MailerAdditionalHeaders(self):
        """ Test mailer with dummy_send """

        mailer = get_actions(self.ff1)['mailer']

        data = {'topic': 'test subject', 'comments': 'test comments'}
        request = self.LoadRequestForm(**data)

        mailer.additional_headers = [
            'Generator: Plone',
            'Token:   abc  ',
        ]

        mailer.onSuccess(data, request)

        self.assertIn('Generator: Plone', self.messageText)
        self.assertIn('Token: abc', self.messageText)
        self.assertIn('To: mdummy@address.com', self.messageText)
        self.assertIn(
            'Subject: =?utf-8?q?test_subject?=', self.messageText)
        msg = email.message_from_string(self.messageText)
        self.assertIn(
            'test comments', msg.get_payload(decode=True))

    def test_MailerLongSubject(self):
        """ Test mailer with subject line > 76 chars (Tracker # 84) """

        long_subject = (
            'Now is the time for all good persons to come to'
            'the aid of the quick brown fox.'
        )

        mailer = get_actions(self.ff1)['mailer']

        data = {'topic': long_subject}
        request = self.LoadRequestForm(**data)
        mailer.onSuccess(data, request)

        msg = email.message_from_string(self.messageText)
        encoded_subject_header = msg['subject']
        decoded_header = email.Header.decode_header(
            encoded_subject_header)[0][0]

        self.assertEqual(decoded_header, long_subject)

    def test_SubjectDollarReplacement(self):
        """
        Simple subject lines should do ${identifier} replacement from
        request.form -- but only for a basic override.
        """
        mailer = get_actions(self.ff1)['mailer']
        mailer.msg_subject = 'This is my ${topic} now'
        data = dict(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )

        # baseline unchanged
        request = self.LoadRequestForm(**data)
        self.messageText = ''
        mailer.onSuccess(data, request)
        self.assertTrue(self.messageText.find(
            'Subject: =?utf-8?q?test_subject?=') > 0)

        data2 = dict(
            topic='test ${subject}',
            replyto='test@test.org',
            comments='test comments'
        )

        # no substitution on field replacement (default situation)
        request = self.LoadRequestForm(**data2)
        self.messageText = ''
        mailer.onSuccess(data2, request)
        self.assertIn(
            'Subject: =?utf-8?q?test_=24=7Bsubject=7D?=', self.messageText)

        # we should get substitution in a basic override
        mailer.subject_field = ''
        request = self.LoadRequestForm(**data)
        self.messageText = ''
        mailer.onSuccess(data, request)
        self.assertIn(
            'Subject: =?utf-8?q?This_is_my_test_subject_now?=',
            self.messageText)

        # we should get substitution in a basic override
        mailer.msg_subject = 'This is my ${untopic} now'
        self.messageText = ''
        mailer.onSuccess(data, request)
        self.assertIn(
            'Subject: =?utf-8?q?This_is_my_=3F=3F=3F_now?=',
            self.messageText)

        # we don't want substitution on user input
        request = self.LoadRequestForm(**data2)
        self.messageText = ''
        mailer.onSuccess(data2, request)
        self.assertIn(
            'Subject: =?utf-8?q?This_is_my_=3F=3F=3F_now?=',
            self.messageText)

    def test_TemplateReplacement(self):
        """
        Mail template prologues, epilogues and footers should do ${identifier}
        replacement from request.form -- this is simpler because there are no
        overrides.
        """

        mailer = get_actions(self.ff1)['mailer']
        mailer.body_pre = 'Hello ${topic},'
        mailer.body_post = 'Thanks, ${topic}!'
        mailer.body_footer = 'Eat my footer, ${topic}.'
        data = dict(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )

        # we should get substitution
        request = self.LoadRequestForm(**data)
        self.messageText = ''
        mailer.onSuccess(data, request)
        self.assertIn('Hello test subject,', self.messageBody)
        self.assertIn('Thanks, test subject!', self.messageBody)
        self.assertIn('Eat my footer, test subject.', self.messageBody)

    def test_UTF8Subject(self):
        """ Test mailer with uft-8 encoded subject line """

        utf8_subject = 'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'
        data = {'topic': utf8_subject}

        mailer = get_actions(self.ff1)['mailer']
        # fields = self.ff1._getFieldObjects()
        request = self.LoadRequestForm(**data)
        mailer.onSuccess(data, request)

        msg = email.message_from_string(self.messageText)
        encoded_subject_header = msg['subject']
        decoded_header = email.Header.decode_header(
            encoded_subject_header)[0][0]

        self.assertEqual(decoded_header, utf8_subject)

    def test_UnicodeSubject(self):
        """ Test mailer with Unicode encoded subject line """
        utf8_subject = 'Effacer les entr\xc3\xa9es sauvegard\xc3\xa9es'
        unicode_subject = utf8_subject.decode('UTF-8')
        data = {'topic': unicode_subject}

        mailer = get_actions(self.ff1)['mailer']
        request = self.LoadRequestForm(**data)
        mailer.onSuccess(data, request)

        msg = email.message_from_string(self.messageText)
        encoded_subject_header = msg['subject']
        decoded_header = email.Header.decode_header(
            encoded_subject_header)[0][0]

        self.assertEqual(decoded_header, utf8_subject)

    def test_Utf8ListSubject(self):
        """ Test mailer with Unicode encoded subject line """
        utf8_subject_list = [
            'Effacer les entr\xc3\xa9es',
            'sauvegard\xc3\xa9es'
        ]
        data = {'topic': utf8_subject_list}
        mailer = get_actions(self.ff1)['mailer']
        request = self.LoadRequestForm(**data)
        mailer.onSuccess(data, request)

        msg = email.message_from_string(self.messageText)
        encoded_subject_header = msg['subject']
        decoded_header = email.Header.decode_header(
            encoded_subject_header)[0][0]

        self.assertEqual(
            decoded_header,
            ', '.join(utf8_subject_list))

    def test_MailerOverrides(self):
        """ Test mailer override functions """

        mailer = get_actions(self.ff1)['mailer']
        mailer.subjectOverride = "python: '{0} and {1}'.format('eggs', 'spam')"
        mailer.senderOverride = 'string: spam@eggs.com'
        mailer.recipientOverride = 'string: eggs@spam.com'
        data = {'topic': 'test subject'}
        request = self.LoadRequestForm(**data)

        mailer.onSuccess(data, request)
        self.assertIn('Subject: =?utf-8?q?eggs_and_spam?=', self.messageText)
        self.assertIn('From: spam@eggs.com', self.messageText)
        self.assertIn('To: eggs@spam.com', self.messageText)

    def test_MailerOverridesWithFieldValues(self):
        mailer = get_actions(self.ff1)['mailer']
        mailer.subjectOverride = 'fields/topic'
        mailer.recipientOverride = 'fields/replyto'
        data = {'topic': 'eggs and spam', 'replyto': u'test@test.ts'}
        request = self.LoadRequestForm(**data)
        mailer.onSuccess(data, request)

        self.assertIn('Subject: =?utf-8?q?eggs_and_spam?=', self.messageText)
        self.assertIn('To: test@test.ts', self.messageText)

    def testMultiRecipientOverrideByString(self):
        """ try multiple recipients in recipient override """

        mailer = get_actions(self.ff1)['mailer']
        mailer.recipientOverride = 'string: eggs@spam.com, spam@spam.com'

        data = {'topic': 'test subject'}
        request = self.LoadRequestForm(**data)
        mailer.onSuccess(data, request)

        self.assertIn('To: eggs@spam.com, spam@spam.com', self.messageText)

    def testMultiRecipientOverrideByTuple(self):
        """ try multiple recipients in recipient override """

        mailer = get_actions(self.ff1)['mailer']
        mailer.recipientOverride = "python: ('eggs@spam.com', 'spam.spam.com')"

        data = {'topic': 'test subject'}
        request = self.LoadRequestForm(**data)
        mailer.onSuccess(data, request)

        self.assertIn('To: eggs@spam.com, spam.spam.com', self.messageText)

    def testRecipientFromRequest(self):
        """ try recipient from designated field  """

        mailer = get_actions(self.ff1)['mailer']
        mailer.to_field = 'replyto'
        mailer.replyto_field = None

        fields = {'topic': 'test subject', 'replyto': 'eggs@spamandeggs.com'}

        request = self.LoadRequestForm(**fields)
        mailer.onSuccess(fields, request)

        self.assertIn('To: eggs@spamandeggs.com', self.messageText)

        fields = {'topic': 'test subject',
                  'replyto': ['eggs@spam.com', 'spam@spam.com']}
        request = self.LoadRequestForm(**fields)
        mailer.onSuccess(fields, request)

        self.assertTrue(self.messageText.find(
            'To: eggs@spam.com, spam@spam.com') > 0)

    def setExecCondition(self, value):
        actions = get_actions(self.ff1)
        IActionExtender(actions['mailer']).execCondition = value
        set_actions(self.ff1, actions)

    def test_ExecConditions(self):
        """ Test mailer with various exec conditions """

        # if an action adapter's execCondition is filled in and evaluates
        # false, the action adapter should not fire.

        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance

        fields = dict(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )
        self.LoadRequestForm(**fields)

        self.messageText = ''
        self.setExecCondition('python: False')
        form.processActions(fields)
        self.assertTrue(len(self.messageText) == 0)

        self.messageText = ''
        self.setExecCondition('python: True')
        form.processActions(fields)
        self.assertTrue(len(self.messageText) > 0)

        self.messageText = ''
        self.setExecCondition('python: 1==0')
        form.processActions(fields)
        self.assertTrue(len(self.messageText) == 0)

        # make sure an empty execCondition causes the action to fire
        self.messageText = ''
        self.setExecCondition('')
        form.processActions(fields)
        self.assertTrue(len(self.messageText) > 0)

    def test_selectiveFieldMailing(self):
        """ Test selective inclusion of fields in the mailing """

        mailer = get_actions(self.ff1)['mailer']
        fields = dict(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )
        request = self.LoadRequestForm(**fields)

        # make sure all fields are sent unless otherwise specified
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertTrue(
            'te=\nst subject' in self.messageBody and
            'test@test.org' in self.messageBody and
            'test comments' in self.messageBody
        )

        # setting some show fields shouldn't change that
        mailer.showFields = ('topic', 'comments',)
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertTrue(
            'te=\nst subject' in self.messageBody and
            'test@test.org' in self.messageBody and
            'test comments' in self.messageBody
        )

        # until we turn off the showAll flag
        mailer.showAll = False
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertTrue(
            'te=\nst subject' in self.messageBody and
            'test@test.org' not in self.messageBody and
            'test comments' in self.messageBody
        )

        # check includeEmpties
        mailer.includeEmpties = False

        # first see if everything's still included
        mailer.showAll = True
        self.messageText = ''
        mailer.onSuccess(fields, request)
        # look for labels
        self.assertTrue(
            self.messageBody.find('Subject') > 0 and
            self.messageBody.find('Your E-Mail Address') > 0 and
            self.messageBody.find('Comments') > 0
        )

        # now, turn off required for a field and leave it empty
        fields = get_schema(self.ff1)
        fields['comments'].required = False
        set_fields(self.ff1, fields)
        fields = {'topic': 'test subject', 'replyto': 'test@test.org'}
        request = self.LoadRequestForm(**fields)
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertIn('Subject', self.messageBody)
        self.assertIn('Your E-Mail Address', self.messageBody)
        self.assertNotIn('Comments', self.messageBody)

    def test_ccOverride(self):
        """ Test override for CC field """

        mailer = get_actions(self.ff1)['mailer']
        fields = dict(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )
        request = self.LoadRequestForm(**fields)
        mailer.cc_recipients = 'test@testme.com'
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertIn('test@testme.com', self.mto)

        # simple override
        mailer.ccOverride = 'string:test@testme.com'
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertIn('test@testme.com', self.mto)

        # list override
        mailer.ccOverride = "python:['test@testme.com', 'test1@testme.com']"
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertTrue(
            'test@testme.com' in self.mto and
            'test1@testme.com' in self.mto
        )

    def test_bccOverride(self):
        """ Test override for BCC field """
        mailer = get_actions(self.ff1)['mailer']
        fields = dict(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )
        request = self.LoadRequestForm(**fields)
        mailer.bcc_recipients = 'test@testme.com'
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertIn('test@testme.com', self.mto)

        # simple override
        mailer.bccOverride = 'string:test@testme.com'
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertIn('test@testme.com', self.mto)

        # list override
        mailer.bccOverride = "python:['test@testme.com', 'test1@testme.com']"
        self.messageText = ''
        mailer.onSuccess(fields, request)
        self.assertTrue(
            'test@testme.com' in self.mto and
            'test1@testme.com' in self.mto
        )

    def testNoRecipient(self):
        """
        This is not the feature in easyform as we need recipient in the mailer
        for easyforms. It will not take the default site's recipient.
        """
        mailer = get_actions(self.ff1)['mailer']
        mailer.recipient_email = u''
        mailer.to_field = None
        mailer.replyto_field = None
        fields = dict(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )

        request = self.LoadRequestForm(**fields)

        self.messageText = ''
        self.assertRaises(ValueError, mailer.onSuccess, fields, request)

    def test_custom_email_template(self):
        """ Test mailer with custom template """
        default_fields = api.content.create(
            self.portal, 'File', id='easyform_mail_body_default.pt')
        default_fields.file = NamedFile('Custom e-mail template!')

        mailer = get_actions(self.ff1)['mailer']
        mailer.onSuccess({}, self.layer['request'])
        self.assertIn(u'Custom e-mail template!', self.messageText)

    def test_MailerXMLAttachments(self):
        """ Test mailer with dummy_send """
        mailer = get_actions(self.ff1)['mailer']
        mailer.sendXML = True
        mailer.sendCSV = False
        fields = dict(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments',
            choices=set(['A', 'B']),
            richtext=RichTextValue(raw='Raw')
        )
        request = self.LoadRequestForm(**fields)
        attachments = mailer.get_attachments(fields, request)
        self.assertEqual(1, len(attachments))



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from collections import OrderedDict
from collective.easyform import easyformMessageFactory as _
from collective.easyform.api import dollar_replacer
from collective.easyform.api import filter_fields
from collective.easyform.api import get_actions
from collective.easyform.api import get_expression
from collective.easyform.api import get_schema
from collective.easyform.interfaces import IActionExtender
from collective.easyform.interfaces import IEasyFormForm
from collective.easyform.interfaces import IEasyFormThanksPage
from collective.easyform.interfaces import IFieldExtender
from collective.easyform.interfaces import ISaveData
from logging import getLogger
from os.path import splitext
from plone.app.z3cform.inline_validation import InlineValidationView
from plone.autoform.form import AutoExtensibleForm
from plone.namedfile.interfaces import INamed
from plone.z3cform import layout
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form import form
from z3c.form.interfaces import DISPLAY_MODE
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import IErrorViewSnippet
from zope.component import getMultiAdapter
from zope.i18nmessageid import MessageFactory
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.schema import getFieldsInOrder
from zope.schema import ValidationError
from ZPublisher.mapply import mapply


logger = getLogger('collective.easyform')
PMF = MessageFactory('plone')


@implementer(IEasyFormForm)
class EasyFormForm(AutoExtensibleForm, form.Form):
    """
    EasyForm form
    """
    form_template = ViewPageTemplateFile('easyform_form.pt')
    thank_you_template = ViewPageTemplateFile('thank_you.pt')
    ignoreContext = True
    css_class = 'easyformForm'
    thanksPage = False

    # allow prefill - see Products/CMFPlone/patches/z3c_form
    allow_prefill_from_GET_request = True

    @property
    def method(self):
        return self.context.method

    @property
    def default_fieldset_label(self):
        return (
            self.context.default_fieldset_label or
            super(EasyFormForm, self).default_fieldset_label
        )

    def action(self):
        """ Redefine <form action=''> attribute.
        """
        action = getattr(self.context, 'formActionOverride')
        if action:
            action = get_expression(self.context, action)
        if not action:
            action = self.context.absolute_url()
        if self.context.forceSSL:
            action = action.replace('http://', 'https://')
        return action

    @property
    def enable_form_tabbing(self):
        return self.context.form_tabbing

    @property
    def enable_unload_protection(self):
        return self.context.unload_protection

    @property
    def enableCSRFProtection(self):
        return self.context.CSRFProtection

    @property
    def label(self):
        return (
            self.thanksPage and
            self.context.thankstitle or
            self.context.Title()
        )

    @property
    def description(self):
        return (
            self.thanksPage and
            self.context.thanksdescription or
            self.context.Description()
        )

    @property
    def schema(self):
        return get_schema(self.context)

    def updateServerSideData(self, data):
        for fname in self.schema:
            field = self.schema[fname]
            efield = IFieldExtender(field)
            serverSide = getattr(efield, 'serverSide', False)
            if not serverSide:
                continue
            fdefault = field.default
            TDefault = getattr(efield, 'TDefault', None)
            value = get_expression(
                self.context, TDefault) if TDefault else fdefault
            data[fname] = value
        return data

    def processActions(self, fields):
        # get a list of adapters with no duplicates, retaining order
        actions = getFieldsInOrder(get_actions(self.context))
        for name, action in actions:
            if not action.required:
                continue
            # Now, see if we should execute it.
            # Check to see if execCondition exists and has contents
            execCondition = IActionExtender(action).execCondition
            if execCondition:
                doit = get_expression(self.context, execCondition)
            else:
                doit = True
            if doit and hasattr(action, 'onSuccess'):
                result = action.onSuccess(fields, self.request)
                if isinstance(result, dict) and len(result):
                    return result

    def setErrorsMessage(self, errors):
        for field in errors:
            if field not in self.widgets:
                continue
            error = ValidationError()
            error.doc = lambda: errors[field]
            view = getMultiAdapter(
                (error, self.request, self.widgets[
                    field], self.widgets[field].field, self, self.context),
                IErrorViewSnippet)
            view.update()
            self.widgets.errors += (view,)
            self.widgets[field].error = view
        self.status = self.formErrorsMessage

    @button.buttonAndHandler(
        PMF(u'Submit'),
        name='submit',
        condition=lambda form: not form.thanksPage
    )
    def handleSubmit(self, action):
        unsorted_data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        unsorted_data = self.updateServerSideData(unsorted_data)
        errors = self.processActions(unsorted_data)
        if errors:
            return self.setErrorsMessage(errors)
        data = OrderedDict(
            [x for x in getFieldsInOrder(self.schema) if x[0] in unsorted_data]
        )
        data.update(unsorted_data)
        thanksPageOverride = self.context.thanksPageOverride
        if not thanksPageOverride:
            # we come back to the form itself.
            # the thanks page is handled in the __call__ method
            return
        thanksPageOverrideAction = self.context.thanksPageOverrideAction
        thanksPage = get_expression(self.context, thanksPageOverride)
        if thanksPageOverrideAction == 'redirect_to':
            self.request.response.redirect(thanksPage)
            return
        if thanksPageOverrideAction == 'traverse_to':
            thanksPage = self.context.restrictedTraverse(
                thanksPage.encode('utf-8'))
            thanksPage = mapply(
                thanksPage,
                self.request.args,
                self.request
            ).encode('utf-8')
            self.request.response.write(thanksPage)

    @button.buttonAndHandler(
        _(u'Reset'),
        name='reset',
        condition=lambda form: form.context.useCancelButton
    )
    def handleReset(self, action):
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        return self.context.absolute_url()

    def setOmitFields(self, fields):
        omit = []
        new_fields = []
        for fname, field in fields.items():
            efield = IFieldExtender(field.field)
            TEnabled = getattr(efield, 'TEnabled', None)
            serverSide = getattr(efield, 'serverSide', False)
            if (
                TEnabled and
                not get_expression(self.context, TEnabled) or
                serverSide
            ):
                omit.append(fname)
            if getattr(efield, 'THidden', False):
                field.mode = HIDDEN_MODE
                field.field.ignoreRequest = False
            new_fields.append(field)
        fields = fields.__class__(*new_fields)
        if omit:
            fields = fields.omit(*omit)
        return fields

    def setThanksFields(self, fields, data):
        omit = filter_fields(self.context, self.schema, data, omit=True)
        new_fields = []
        for fname, field in fields.items():
            if field.mode == HIDDEN_MODE:
                field.mode = DISPLAY_MODE
            new_fields.append(field)
        fields = fields.__class__(*new_fields)
        if omit:
            fields = fields.omit(*omit)

        return fields

    def updateFields(self):
        if self.thanksPage:
            return
        super(EasyFormForm, self).updateFields()
        if not hasattr(self, 'base_fields'):
            self.base_fields = self.fields
        if not hasattr(self, 'base_groups'):
            self.base_groups = dict([(i.label, i.fields) for i in self.groups])
        self.fields = self.setOmitFields(self.base_fields)
        for group in self.groups:
            group.fields = self.setOmitFields(
                self.base_groups.get(group.label))

    def updateActions(self):
        super(EasyFormForm, self).updateActions()
        if 'submit' in self.actions:
            if self.context.submitLabelOverride:
                self.actions['submit'].title = get_expression(
                    self.context, self.context.submitLabelOverride)
            else:
                self.actions['submit'].title = self.context.submitLabel
        if 'reset' in self.actions:
            self.actions['reset'].title = self.context.resetLabel

    def formMaybeForceSSL(self):
        """ Redirect to an https:// URL if the 'force SSL' option is on.

            However, don't do so for those with rights to edit the form,
            to avoid making the form uneditable if SSL isn't configured
            properly.  These users will still get an SSL-ified form
            action for when the form is submitted.
        """
        sm = getSecurityManager()
        if (
            self.context.forceSSL and
            not sm.checkPermission('cmf.ModifyPortalContent', self.context)
        ):
            # Make sure we're being accessed via a secure connection
            if self.request['SERVER_URL'].startswith('http://'):
                secure_url = self.request['URL'].replace('http://', 'https://')
                self.request.response.redirect(
                    secure_url, status='movedtemporarily')

    def update(self):
        """ Update form - see interfaces.IForm """
        self.formMaybeForceSSL()
        super(EasyFormForm, self).update()
        self.template = self.form_template
        if (
            self.request.method != 'POST' or
            self.context.thanksPageOverride
        ):
            # go with all but default thank you page rendering
            return
        data, errors = self.extractData()
        if errors:
            # render errors
            return
        data = self.updateServerSideData(data)
        self.thanksPage = True
        self.template = self.thank_you_template
        self.fields = self.setThanksFields(self.base_fields, data)
        for name in list(self.widgets.keys()):
            if name not in self.fields:
                del self.widgets[name]
        self.widgets.mode = self.mode = DISPLAY_MODE
        self.widgets.update()

        for group in self.groups:
            group.fields = self.setThanksFields(
                self.base_groups.get(group.label),
                data
            )
            for name in list(group.widgets.keys()):
                if name not in group.fields:
                    del group.widgets[name]
            group.widgets.mode = DISPLAY_MODE
            group.widgets.update()
        prologue = self.context.thanksPrologue
        epilogue = self.context.thanksEpilogue
        self.thanksPrologue = (
            prologue and
            dollar_replacer(prologue.output, data)
        )
        self.thanksEpilogue = (
            epilogue and
            dollar_replacer(epilogue.output, data)
        )
        alsoProvides(self.request, IEasyFormThanksPage)


EasyFormView = layout.wrap_form(EasyFormForm)


class EasyFormFormEmbedded(EasyFormForm):

    """
    EasyForm form embedded
    """
    form_template = ViewPageTemplateFile('easyform_form_embedded.pt')


class EasyFormInlineValidationView(InlineValidationView):

    def __call__(self, fname=None, fset=None):
        self.context = EasyFormForm(self.context, self.request)
        return super(EasyFormInlineValidationView, self).__call__(fname, fset)


class GetSaveDataAdaptersView(BrowserView):

    def __call__(self, *args, **kwargs):
        """return all contained save data adapters"""
        view = EasyFormForm(self.context, self.request)
        form = view.context
        adapters = []
        actions = get_actions(form)
        for action_id in actions:
            action = actions[action_id]
            if ISaveData.providedBy(action):
                adapters.append(action)
        return adapters


class ValidateFile(BrowserView):

    def __call__(self, value, size=1048576, allowed_types=None,
                 forbidden_types=None):
        if not value:
            return False
        if not INamed.providedBy(value):
            return False
        if size and value.getSize() > size:
            return _(
                'msg_file_too_big',
                mapping={'size': size},
                default=u'File is bigger than allowed size of ${size} bytes!'
            )
        ftype = splitext(value.filename)[-1]
        # remove leading dot '.' from file extension
        ftype = ftype and ftype[1:].lower() or ''
        if allowed_types and ftype not in allowed_types:
            return _(
                'msg_file_not_allowed',
                mapping={'ftype': ftype.upper()},
                default=u'File type "${ftype}" is not allowed!'
            )
        if forbidden_types and ftype in forbidden_types:
            return _(
                'msg_file_not_allowed',
                mapping={'ftype': ftype.upper()},
                default=u'File type "${ftype}" is not allowed!'
            )
        return False

# BBB
ValidateFileSize = ValidateFile



# -*- coding: utf-8 -*-
from .validators import isTALES
from collective.easyform import easyformMessageFactory as _  # NOQA
from collective.easyform import config
from collective.easyform.interfaces import IAction
from plone import api
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.schema import Email
from plone.supermodel.model import fieldset
from Products.CMFPlone.utils import safe_unicode
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.browser.textarea import TextAreaWidget

import zope.i18nmessageid
import zope.interface
import zope.schema.interfaces


PMF = zope.i18nmessageid.MessageFactory('plone')
MODIFY_PORTAL_CONTENT = 'cmf.ModifyPortalContent'


def default_mail_body():
    """ Default mail body for mailer action

        Acquire 'mail_body_default.pt' or return hard coded default
    """
    portal = api.portal.get()
    mail_body_default = portal.restrictedTraverse(
        'easyform_mail_body_default.pt', default=None)
    if mail_body_default:
        return safe_unicode(mail_body_default.file.data)
    else:
        return config.MAIL_BODY_DEFAULT


class IMailer(IAction):
    """A form action adapter that will e-mail form input."""

    directives.write_permission(
        recipient_name=config.EDIT_ADDRESSING_PERMISSION)
    directives.read_permission(recipient_name=MODIFY_PORTAL_CONTENT)
    recipient_name = zope.schema.TextLine(
        title=_(u'label_formmailer_recipient_fullname',
                default=u"Recipient's full name"),
        description=_(
            u'help_formmailer_recipient_fullname',
            default=u'The full name of the recipient of the mailed form.'
        ),
        default=u'',
        missing_value=u'',
        required=False,
    )

    directives.write_permission(
        recipient_email=config.EDIT_ADDRESSING_PERMISSION)
    directives.read_permission(recipient_email=MODIFY_PORTAL_CONTENT)
    recipient_email = Email(
        title=_(u'label_formmailer_recipient_email',
                default=u"Recipient's e-mail address"),
        description=_(u'help_formmailer_recipient_email',
                      default=u'The recipients e-mail address.'),
        required=False,
    )
    fieldset(u'addressing', label=_('Addressing'), fields=[
             'to_field', 'cc_recipients', 'bcc_recipients', 'replyto_field'])
    directives.write_permission(to_field=config.EDIT_ADVANCED_PERMISSION)
    directives.read_permission(to_field=MODIFY_PORTAL_CONTENT)
    to_field = zope.schema.Choice(
        title=_(u'label_formmailer_to_extract',
                default=u'Extract Recipient From'),
        description=_(
            u'help_formmailer_to_extract',
            default=u'Choose a form field from which you wish to extract '
                    u'input for the To header. If you choose anything other '
                    u'than "None", this will override the "Recipient\'s " '
                    u'e-mail address setting above. Be very cautious about '
                    u'allowing unguarded user input for this purpose.'
        ),
        required=False,
        vocabulary='easyform.Fields',
    )

    directives.write_permission(
        cc_recipients=config.EDIT_ADDRESSING_PERMISSION)
    directives.read_permission(cc_recipients=MODIFY_PORTAL_CONTENT)
    cc_recipients = zope.schema.Text(
        title=_(u'label_formmailer_cc_recipients',
                default=u'CC Recipients'),
        description=_(
            u'help_formmailer_cc_recipients',
            default=u'E-mail addresses which receive a carbon copy.'
        ),
        default=u'',
        missing_value=u'',
        required=False,
    )

    directives.write_permission(
        bcc_recipients=config.EDIT_ADDRESSING_PERMISSION)
    directives.read_permission(bcc_recipients=MODIFY_PORTAL_CONTENT)
    bcc_recipients = zope.schema.Text(
        title=_(u'label_formmailer_bcc_recipients',
                default=u'BCC Recipients'),
        description=_(
            u'help_formmailer_bcc_recipients',
            default=u'E-mail addresses which receive a blind carbon copy.'
        ),
        default=u'',
        missing_value=u'',
        required=False,
    )

    directives.write_permission(replyto_field=config.EDIT_ADVANCED_PERMISSION)
    directives.read_permission(replyto_field=MODIFY_PORTAL_CONTENT)
    replyto_field = zope.schema.Choice(
        title=_(u'label_formmailer_replyto_extract',
                default=u'Extract Reply-To From'),
        description=_(
            u'help_formmailer_replyto_extract',
            default=u'Choose a form field from which you wish to extract '
                    u'input for the Reply-To header. NOTE: You should '
                    u'activate e-mail address verification for the '
                    u'designated field.'
        ),
        required=False,
        vocabulary='easyform.Fields',
    )
    fieldset(u'message', label=PMF('Message'), fields=[
             'msg_subject', 'subject_field', 'body_pre', 'body_post',
             'body_footer', 'showAll', 'showFields', 'includeEmpties',
             'sendCSV', 'sendXML'])
    directives.read_permission(msg_subject=MODIFY_PORTAL_CONTENT)
    msg_subject = zope.schema.TextLine(
        title=_(u'label_formmailer_subject', default=u'Subject'),
        description=_(u'help_formmailer_subject', default=u''
                      u'Subject line of message. This is used if you '
                      u'do not specify a subject field or if the field '
                      u'is empty.'),
        default=u'Form Submission',
        missing_value=u'',
        required=False,
    )

    directives.write_permission(subject_field=config.EDIT_ADVANCED_PERMISSION)
    directives.read_permission(subject_field=MODIFY_PORTAL_CONTENT)
    subject_field = zope.schema.Choice(
        title=_(u'label_formmailer_subject_extract',
                default=u'Extract Subject From'),
        description=_(u'help_formmailer_subject_extract', default=u''
                      u'Choose a form field from which you wish to extract '
                      u'input for the mail subject line.'),
        required=False,
        vocabulary='easyform.Fields',
    )
    directives.read_permission(body_pre=MODIFY_PORTAL_CONTENT)
    directives.widget('body_pre', TextAreaWidget)

    body_pre = RichText(
        title=_(u'label_formmailer_body_pre', default=u'Body (prepended)'),
        description=_(u'help_formmailer_body_pre',
                      default=u'Text prepended to fields listed in mail-body'),
        default=u'',
        missing_value=u'',
        default_mime_type='text/x-web-intelligent',
        allowed_mime_types=('text/x-web-intelligent', ),
        output_mime_type='text/x-html-safe',
        required=False,
    )
    directives.read_permission(body_post=MODIFY_PORTAL_CONTENT)
    directives.widget('body_post', TextAreaWidget)
    body_post = RichText(
        title=_(u'label_formmailer_body_post', default=u'Body (appended)'),
        description=_(u'help_formmailer_body_post',
                      default=u'Text appended to fields listed in mail-body'),
        default=u'',
        missing_value=u'',
        default_mime_type='text/x-web-intelligent',
        allowed_mime_types=('text/x-web-intelligent', ),
        output_mime_type='text/x-html-safe',
        required=False,
    )
    directives.read_permission(body_footer=MODIFY_PORTAL_CONTENT)
    directives.widget('body_footer', TextAreaWidget)
    body_footer = RichText(
        title=_(u'label_formmailer_body_footer',
                default=u'Body (signature)'),
        description=_(u'help_formmailer_body_footer',
                      default=u'Text used as the footer at '
                      u'bottom, delimited from the body by a dashed line.'),
        default=u'',
        missing_value=u'',
        default_mime_type='text/x-web-intelligent',
        allowed_mime_types=('text/x-web-intelligent', ),
        output_mime_type='text/x-html-safe',
        required=False,
    )

    directives.read_permission(showAll=MODIFY_PORTAL_CONTENT)
    showAll = zope.schema.Bool(
        title=_(u'label_mailallfields_text', default=u'Include All Fields'),
        description=_(u'help_mailallfields_text', default=u''
                      u'Check this to include input for all fields '
                      u'(except label and file fields). If you check '
                      u'this, the choices in the pick box below '
                      u'will be ignored.'),
        default=True,
        required=False,
    )

    directives.read_permission(showFields=MODIFY_PORTAL_CONTENT)
    showFields = zope.schema.List(
        title=_(u'label_mailfields_text', default=u'Show Responses'),
        description=_(
            u'help_mailfields_text',
            default=u'Pick the fields whose inputs you\'d like to include in '
                    u'the e-mail.'
        ),
        unique=True,
        required=False,
        value_type=zope.schema.Choice(vocabulary='easyform.Fields'),
    )

    directives.read_permission(includeEmpties=MODIFY_PORTAL_CONTENT)
    includeEmpties = zope.schema.Bool(
        title=_(u'label_mailEmpties_text', default=u'Include Empties'),
        description=_(u'help_mailEmpties_text', default=u''
                      u'Check this to include titles '
                      u'for fields that received no input. Uncheck '
                      u'to leave fields with no input out of the e-mail.'),
        default=True,
        required=False,
    )

    directives.read_permission(sendCSV=MODIFY_PORTAL_CONTENT)
    sendCSV = zope.schema.Bool(
        title=_(u'label_sendCSV_text', default=u'Send CSV data attachment'),
        description=_(u'help_sendCSV_text', default=u''
                      u'Check this to send a CSV file '
                      u'attachment containing the values '
                      u'filled out in the form.'),
        default=False,
        required=False,
    )

    directives.read_permission(sendXML=MODIFY_PORTAL_CONTENT)
    sendXML = zope.schema.Bool(
        title=_(u'label_sendXML_text', default=u'Send XML data attachment'),
        description=_(u'help_sendXML_text', default=u''
                      u'Check this to send an XML file '
                      u'attachment containing the values '
                      u'filled out in the form.'),
        default=False,
        required=False,
    )

    fieldset(u'template', label=PMF(
        'Template'), fields=['body_pt', 'body_type'])
    directives.write_permission(body_pt=config.EDIT_TALES_PERMISSION)
    directives.read_permission(body_pt=MODIFY_PORTAL_CONTENT)
    body_pt = zope.schema.Text(
        title=_(u'label_formmailer_body_pt', default=u'Mail-Body Template'),
        description=_(
            u'help_formmailer_body_pt',
            default=u'This is a Zope Page Template used for rendering of '
                    u'the mail-body. You don\'t need to modify it, but if you '
                    u'know TAL (Zope\'s Template Attribute Language) have '
                    u'the full power to customize your outgoing mails.'
        ),
        defaultFactory=default_mail_body,
        missing_value=u'',
    )

    directives.write_permission(body_type=config.EDIT_ADVANCED_PERMISSION)
    directives.read_permission(body_type=MODIFY_PORTAL_CONTENT)
    body_type = zope.schema.Choice(
        title=_(u'label_formmailer_body_type', default=u'Mail Format'),
        description=_(
            u'help_formmailer_body_type',
            default=u'Set the mime-type of the mail-body. Change this '
                    u'setting only if you know exactly what you are doing. '
                    u'Leave it blank for default behaviour.'
        ),
        default=u'html',
        vocabulary='easyform.MimeList',
    )
    fieldset(u'headers', label=_('Headers'),
             fields=['xinfo_headers', 'additional_headers'])
    directives.widget(xinfo_headers=CheckBoxFieldWidget)
    directives.write_permission(xinfo_headers=config.EDIT_ADVANCED_PERMISSION)
    directives.read_permission(xinfo_headers=MODIFY_PORTAL_CONTENT)
    xinfo_headers = zope.schema.List(
        title=_(u'label_xinfo_headers_text', default=u'HTTP Headers'),
        description=_(u'help_xinfo_headers_text', default=u''
                      u'Pick any items from the HTTP headers that '
                      u'you\'d like to insert as X- headers in the message.'),
        unique=True,
        required=False,
        default=[u'HTTP_X_FORWARDED_FOR', u'REMOTE_ADDR', u'PATH_INFO'],
        missing_value=[u'HTTP_X_FORWARDED_FOR', u'REMOTE_ADDR', u'PATH_INFO'],
        value_type=zope.schema.Choice(vocabulary='easyform.XinfoHeaders'),
    )
    directives.write_permission(
        additional_headers=config.EDIT_ADVANCED_PERMISSION)
    directives.read_permission(additional_headers=MODIFY_PORTAL_CONTENT)
    additional_headers = zope.schema.List(
        title=_(u'label_formmailer_additional_headers',
                default=u'Additional Headers'),
        description=_(
            u'help_formmailer_additional_headers',
            default=u'Additional e-mail-header lines. Only use '
                    u'RFC822-compliant headers.'
        ),
        unique=True,
        required=False,
        value_type=zope.schema.TextLine(
            title=_(u'extra_header',
                    default=u'${name} Header', mapping={u'name': u'HTTP'}),
        ),
    )

    fieldset(
        u'overrides',
        label=_('Overrides'),
        fields=[
            'subjectOverride',
            'senderOverride',
            'recipientOverride',
            'ccOverride',
            'bccOverride'
        ]
    )

    directives.write_permission(subjectOverride=config.EDIT_TALES_PERMISSION)
    directives.read_permission(subjectOverride=MODIFY_PORTAL_CONTENT)
    subjectOverride = zope.schema.TextLine(
        title=_(u'label_subject_override_text', default=u'Subject Expression'),
        description=_(
            u'help_subject_override_text',
            default=u'A TALES expression that will be evaluated to override '
                    u'any value otherwise entered for the e-mail subject '
                    u'header. Leave empty if unneeded. Your expression '
                    u'should evaluate as a string. PLEASE NOTE: errors in '
                    u'the evaluation of this expression will cause an error '
                    u'on form display.'
        ),
        required=False,
        default=u'',
        missing_value=u'',
        constraint=isTALES,
    )

    directives.write_permission(senderOverride=config.EDIT_TALES_PERMISSION)
    directives.read_permission(senderOverride=MODIFY_PORTAL_CONTENT)
    senderOverride = zope.schema.TextLine(
        title=_(u'label_sender_override_text', default=u'Sender Expression'),
        description=_(
            u'help_sender_override_text',
            default=u'A TALES expression that will be evaluated to override '
                    u'the "From" header. Leave empty if unneeded. '
                    u'Your expression should evaluate as a string. '
                    u'PLEASE NOTE: errors in the evaluation of this '
                    u'expression will cause an error on form display.'),
        required=False,
        default=u'',
        missing_value=u'',
        constraint=isTALES,
    )

    directives.write_permission(recipientOverride=config.EDIT_TALES_PERMISSION)
    directives.read_permission(recipientOverride=MODIFY_PORTAL_CONTENT)
    recipientOverride = zope.schema.TextLine(
        title=_(u'label_recipient_override_text',
                default=u'Recipient Expression'),
        description=_(
            u'help_recipient_override_text',
            default=u'A TALES expression that will be evaluated to override '
                    u'any value otherwise entered for the recipient '
                    u'e-mail address. You are strongly cautioned against using'
                    u'unvalidated data from the request for this purpose. '
                    u'Leave empty if unneeded. Your expression should '
                    u'evaluate as a string. PLEASE NOTE: errors in the '
                    u'evaluation of this expression will cause '
                    u'an error on form display.'),
        required=False,
        default=u'',
        missing_value=u'',
        constraint=isTALES,
    )

    directives.write_permission(ccOverride=config.EDIT_TALES_PERMISSION)
    directives.read_permission(ccOverride=MODIFY_PORTAL_CONTENT)
    ccOverride = zope.schema.TextLine(
        title=_(u'label_cc_override_text', default=u'CC Expression'),
        description=_(
            u'help_cc_override_text',
            default=u'A TALES expression that will be evaluated to override '
                    u'any value otherwise entered for the CC list. You are '
                    u'strongly cautioned against using unvalidated data from '
                    u'the request for this purpose. Leave empty if unneeded. '
                    u'Your expression should evaluate as a sequence of '
                    u'strings. PLEASE NOTE: errors in the evaluation of this '
                    u'expression will cause an error on form display.'),
        required=False,
        default=u'',
        missing_value=u'',
        constraint=isTALES,
    )

    directives.write_permission(bccOverride=config.EDIT_TALES_PERMISSION)
    directives.read_permission(bccOverride=MODIFY_PORTAL_CONTENT)
    bccOverride = zope.schema.TextLine(
        title=_(u'label_bcc_override_text', default=u'BCC Expression'),
        description=_(
            u'help_bcc_override_text',
            default=u'A TALES expression that will be evaluated to override '
                    u'any value otherwise entered for the BCC list. '
                    u'You are strongly cautioned against using '
                    u'unvalidated data from the request for this purpose. '
                    u'Leave empty if unneeded. Your expression should '
                    u'evaluate as a sequence of strings. PLEASE NOTE: errors '
                    u'in the evaluation of this expression will cause '
                    u'an error on form display.'),
        required=False,
        default=u'',
        missing_value=u'',
        constraint=isTALES,
    )



# -*- coding: utf-8 -*-
from collective.easyform.tests import base
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from zope.component import getUtility


class AllowedFieldsControlPanelFuncionalTest(base.EasyFormTestCase):
    """Test that changes in the easyform control panel are actually
    stored in the registry.
    """

    def setUpPloneSite(self):
        pass

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(self.app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic ' + SITE_OWNER_NAME + ':' + SITE_OWNER_PASSWORD
        )

    def test_easyform_control_panel_link(self):
        self.browser.open(
            self.portal_url + '/@@overview-controlpanel')
        self.browser.getLink('easyform').click()
        self.assertTrue('easyform Settings' in self.browser.contents)

    def test_easyform_control_panel_backlink(self):
        self.browser.open(
            self.portal_url + '/@@easyform-controlpanel')
        self.assertTrue('General' in self.browser.contents)

    def test_easyform_control_panel_sidebar(self):
        self.browser.open(
            self.portal_url + '/@@navigation-controlpanel')
        self.browser.getLink('Site Setup').click()
        self.assertEqual(
            self.browser.url,
            'http://nohost/plone/@@overview-controlpanel')

    def test_easyform_control_panel_checkbox(self):
        self.browser.open(
            self.portal_url + '/@@easyform-controlpanel')
        self.browser.getControl('Rich Text').selected = False
        self.browser.getControl('Save').click()
        registry = getUtility(IRegistry)
        self.assertNotIn(
            'plone.app.textfield.RichText',
            registry.records['easyform.allowedFields'].value)

# EOF



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from collections import OrderedDict as BaseDict
from collective.easyform.config import MODEL_DEFAULT
from collective.easyform.interfaces import IFieldExtender
from email.utils import formataddr
from plone import api
from plone.namedfile.interfaces import INamedBlobFile
from plone.namedfile.interfaces import INamedFile
from plone.supermodel import loadString
from plone.supermodel import serializeSchema
from plone.supermodel.parser import SupermodelParseError
from Products.CMFCore.Expression import Expression
from Products.CMFCore.Expression import getExprContext
from Products.CMFPlone.utils import safe_unicode
from re import compile
from types import StringTypes
from zope.schema import getFieldsInOrder


CONTEXT_KEY = u'context'
# regular expression for dollar-sign variable replacement.
# we want to find ${identifier} patterns
dollarRE = compile(r'\$\{(.+?)\}')


class OrderedDict(BaseDict):
    """
    A wrapper around dictionary objects that provides an ordering for
    keys() and items().
    """
    security = ClassSecurityInfo()
    security.setDefaultAccess('allow')

    def reverse(self):
        items = list(self.items())
        items.reverse()
        return items


InitializeClass(OrderedDict)


class DollarVarReplacer(object):
    """
    Initialize with a dictionary, then self.sub returns a string
    with all ${key} substrings replaced with values looked
    up from the dictionary.

    >>> from collective.easyform import api

    >>> adict = {'one':'two', '_two':'three', '.two':'four'}
    >>> dvr = api.DollarVarReplacer(adict)

    >>> dvr.sub('one one')
    'one one'

    >>> dvr.sub('one ${one}')
    'one two'

    >>> dvr.sub('one ${two}')
    'one ???'

    Skip any key beginning with _ or .
    >>> dvr.sub('one ${_two}')
    'one ???'

    >>> dvr.sub('one ${.two}')
    'one ???'

    """

    def __init__(self, adict):
        self.adict = adict

    def sub(self, s):
        return dollarRE.sub(self.repl, s)

    def repl(self, mo):
        key = mo.group(1)
        if key and key[0] not in ['_', '.']:
            try:
                return self.adict[mo.group(1)]
            except KeyError:
                pass
        return '???'


def get_expression(context, expression_string, **kwargs):
    """ Get TALES expression

    :param context: [required] TALES expression context
    :param string expression_string: [required] TALES expression string
    :param dict kwargs: additional arguments for expression
    :returns: result of TALES expression
    """
    if isinstance(expression_string, unicode):
        expression_string = expression_string.encode('utf-8')

    expression_context = getExprContext(context, context)
    for key in kwargs:
        expression_context.setGlobal(key, kwargs[key])
    expression = Expression(expression_string)
    return expression(expression_context)


def get_context(field):
    """ Get context of field

    :param field: [required] easyform field
    :returns: easyform
    """
    return field.interface.getTaggedValue(CONTEXT_KEY)


def get_model(data, context):
    schema = None
    # if schema is set on context it has priority
    if data is not None:
        try:
            schema = loadString(data).schema
        except SupermodelParseError:  # pragma: no cover
            pass

    # 2nd we try aquire the model
    if not schema:
        nav_root = api.portal.get_navigation_root(context)
        schema = nav_root.get('easyform_model_default.xml')

    # finally we fall back to the hardcoded example
    if not schema:
        schema = loadString(MODEL_DEFAULT).schema
    schema.setTaggedValue(CONTEXT_KEY, context)
    return schema


# caching this breaks with memcached
def get_schema(context):
    try:
        data = context.fields_model
    except AttributeError:
        data = None
    return get_model(data, context)


# caching this breaks with memcached
def get_actions(context):
    try:
        data = context.actions_model
    except AttributeError:
        data = None
    return get_model(data, context)


def set_fields(context, schema):
    # serialize the current schema
    snew_schema = serializeSchema(schema)
    # store the current schema
    context.fields_model = snew_schema
    context.notifyModified()


def set_actions(context, schema):
    # fix setting widgets
    schema.setTaggedValue('plone.autoform.widgets', {})
    # serialize the current schema
    snew_schema = serializeSchema(schema)
    # store the current schema
    context.actions_model = snew_schema
    context.notifyModified()


def format_addresses(addresses, names=[]):
    """
    Format destination (To) input.
    Input may be a string or sequence of strings;
    returns a well-formatted address string.

    :param addresses: [required] Mail addresses.
    :type addresses: sequence or string
    :param names: Names corresponding to mail addresses.
    :type addresses: sequence or string
    :returns: Well formatted address string.
    :rtype: String

    >>> from collective.easyform.api import format_addresses

    >>> format_addresses('sim@sala.bim')
    'sim@sala.bim'

    >>> format_addresses('sim@sala.bim', 'sim')
    'sim <sim@sala.bim>'

    >>> format_addresses('ähm@öhm.ühm', 'ähm')
    '\xc3\xa4hm <\xc3\xa4hm@\xc3\xb6hm.\xc3\xbchm>'

    >>> format_addresses('sim@sala.bim, hokus@pokus.fidibus')
    'sim@sala.bim, hokus@pokus.fidibus'

    >>> format_addresses('sim@sala.bim\nhokus@pokus.fidibus')
    'sim@sala.bim, hokus@pokus.fidibus'

    >>> format_addresses('sim@sala.bim, hokus@pokus.fidibus', 'sim')
    'sim <sim@sala.bim>, hokus@pokus.fidibus'

    >>> format_addresses('sim@sala.bim, hokus@pokus.fidibus', 'sim, hokus')
    'sim <sim@sala.bim>, hokus <hokus@pokus.fidibus>'

    >>> format_addresses('sim@sala.bim, hokus@pokus.fidibus', 'sim\nhokus')
    'sim <sim@sala.bim>, hokus <hokus@pokus.fidibus>'

    >>> format_addresses(['sim@sala.bim', 'hokus@pokus.fidibus'],
    ...                      ['sim', 'hokus'])
    'sim <sim@sala.bim>, hokus <hokus@pokus.fidibus>'

    >>> format_addresses(('sim@sala.bim', 'hokus@pokus.fidibus'),
    ...                      ('sim', 'hokus'))
    'sim <sim@sala.bim>, hokus <hokus@pokus.fidibus>'

    >>> format_addresses([])
    ''

    >>> format_addresses('')
    ''

    """
    if type(addresses) in StringTypes:
        # replace common separators `;` and `,` with newlines so the
        # splitlines method works
        addresses = addresses.replace(',', '\n').replace(';', '\n')
        addresses = [s.strip() for s in addresses.splitlines()]
    assert(isinstance(addresses, list) or isinstance(addresses, tuple))  # ensure iterable  # noqa
    addresses = [
        safe_unicode(s).strip().encode('utf-8') for s in addresses if s
    ]

    if names and type(names) in StringTypes:
        # replace common separators `;` and `,` with newlines so the
        # splitlines method works
        names = names.replace(',', '\n').replace(';', '\n')
        names = [s for s in names.splitlines()]
    if not names:
        names = []
    assert(isinstance(names, list) or isinstance(names, tuple))
    names = [safe_unicode(s).strip().encode('utf-8') for s in names if s]

    address_pairs = []
    for cnt, address in enumerate(addresses):
        address_pairs.append((
            names[cnt] if len(names) > cnt else False,
            address
        ))
    ret = ', '.join([formataddr(pair) for pair in address_pairs])
    return ret


def dollar_replacer(s, data):
    dr = DollarVarReplacer(data)
    return dr.sub(s)


def lnbr(text):
    """Converts line breaks to html breaks
    """
    return "<br/>".join(text.strip().splitlines()) if text else text


def is_file_data(field):
    """Return True, if field is a file field.
    """
    ifaces = (INamedFile, INamedBlobFile)
    for i in ifaces:
        if i.providedBy(field):
            return True
    return False


def filter_fields(context, schema, unsorted_data, omit=False):
    """Filter according to ``showAll``, ``showFields`` and ``includeEmpties``
    settings to display in result mailings, thanks pages and the like.
    """
    data = OrderedDict([
        (x[0], unsorted_data[x[0]])
        for x in getFieldsInOrder(schema)
        if x[0] in unsorted_data
    ])

    # TODO: Exclude labels
    fields = [
        f for f in data
        if not (is_file_data(data[f])) and not (
            # TODO: serverSide should always be excluded
            # Also, when shoAll = 0 and serverSite = 1, it will be included
            # which is wrong.
            getattr(context, 'showAll', True) and
            IFieldExtender(schema[f]).serverSide
        )
    ]

    if not getattr(context, 'showAll', True):
        showFields = getattr(context, 'showFields', []) or []
        fields = [f for f in fields if f in showFields]

    if not getattr(context, 'includeEmpties', True):
        fields = [f for f in fields if data[f]]

    ret = []
    if omit:
        # return field ids to omit
        ret = [f for f in data if f not in fields]
    else:
        # return field ids and field instances to include
        ret = OrderedDict([(f, data[f]) for f in fields])

    return ret


def filter_widgets(context, widgets):
    """Filter according to ``showAll`` and ``showFields``
    settings to return proper widgets for result mailings,
    thanks pages and the like.
    """
    filtered_widgets = {}
    show_all = getattr(context, 'showAll', True)
    show_fields = getattr(context, 'showFields', []) or []
    for field_id, widget in widgets.items():
        if not show_all:
            if show_fields and field_id in show_fields:
                filtered_widgets[field_id] = widget.render()
        else:
            filtered_widgets[field_id] = widget.render()
    return filtered_widgets



# -*- coding: utf-8 -*-
from collective.easyform.interfaces import IAction
from collective.easyform.interfaces import IActionExtender
from collective.easyform.interfaces import IEasyFormActionsContext
from collective.easyform.interfaces import IEasyFormFieldsContext
from collective.easyform.interfaces import IFieldExtender
from plone.autoform.interfaces import WIDGETS_KEY
from plone.supermodel.parser import IFieldMetadataHandler
from plone.supermodel.utils import ns
from zope.component import adapter
from zope.interface import implementer
from zope.schema.interfaces import IField


@adapter(IEasyFormFieldsContext, IField)
def get_field_extender(context, field):
    return IFieldExtender


def _get_(self, key):
    return self.field.interface.queryTaggedValue(
        key,
        {}
    ).get(self.field.__name__)


def _set_(self, value, key):
    data = self.field.interface.queryTaggedValue(key, {})
    data[self.field.__name__] = value
    self.field.interface.setTaggedValue(key, data)


@implementer(IFieldExtender)
@adapter(IField)
class FieldExtender(object):

    def __init__(self, field):
        self.field = field

    field_widget = property(lambda x: _get_(x, WIDGETS_KEY),
                            lambda x, value: _set_(x, value, WIDGETS_KEY))
    TDefault = property(lambda x: _get_(x, 'TDefault'),
                        lambda x, value: _set_(x, value, 'TDefault'))
    TEnabled = property(lambda x: _get_(x, 'TEnabled'),
                        lambda x, value: _set_(x, value, 'TEnabled'))
    TValidator = property(lambda x: _get_(x, 'TValidator'),
                          lambda x, value: _set_(x, value, 'TValidator'))
    serverSide = property(lambda x: _get_(x, 'serverSide'),
                          lambda x, value: _set_(x, value, 'serverSide'))
    validators = property(lambda x: _get_(x, 'validators'),
                          lambda x, value: _set_(x, value, 'validators'))
    THidden = property(lambda x: _get_(x, 'THidden'),
                       lambda x, value: _set_(x, value, 'THidden'))


@implementer(IFieldMetadataHandler)
class EasyFormFieldMetadataHandler(object):

    """Support the easyform: namespace in model definitions.
    """

    namespace = 'http://namespaces.plone.org/supermodel/easyform'
    prefix = 'easyform'

    def read(self, fieldNode, schema, field):
        name = field.__name__
        for i in ['TDefault', 'TEnabled', 'TValidator']:
            value = fieldNode.get(ns(i, self.namespace))
            if value:
                data = schema.queryTaggedValue(i, {})
                data[name] = value
                schema.setTaggedValue(i, data)
        # serverSide
        value = fieldNode.get(ns('serverSide', self.namespace))
        if value:
            data = schema.queryTaggedValue('serverSide', {})
            data[name] = value.lower() == 'true'
            schema.setTaggedValue('serverSide', data)
        # validators
        value = fieldNode.get(ns('validators', self.namespace))
        if value:
            data = schema.queryTaggedValue('validators', {})
            data[name] = value.split('|')
            schema.setTaggedValue('validators', data)
        # hidden
        value = fieldNode.get(ns('THidden', self.namespace))
        if value:
            data = schema.queryTaggedValue('THidden', {})
            data[name] = value.lower() == 'true'
            schema.setTaggedValue('THidden', data)

    def write(self, fieldNode, schema, field):
        name = field.__name__
        for i in ['TDefault', 'TEnabled', 'TValidator']:
            value = schema.queryTaggedValue(i, {}).get(name, None)
            if value:
                fieldNode.set(ns(i, self.namespace), value)
        # serverSide
        value = schema.queryTaggedValue('serverSide', {}).get(name, None)
        if isinstance(value, bool):
            fieldNode.set(ns('serverSide', self.namespace), str(value))
        # validators
        value = schema.queryTaggedValue('validators', {}).get(name, None)
        if value:
            fieldNode.set(ns('validators', self.namespace), "|".join(value))
        # hidden
        value = schema.queryTaggedValue('THidden', {}).get(name, None)
        if isinstance(value, bool):
            fieldNode.set(ns('THidden', self.namespace), str(value))


@adapter(IEasyFormActionsContext, IAction)
def get_action_extender(context, action):
    return IActionExtender


@implementer(IActionExtender)
@adapter(IAction)
class ActionExtender(object):

    def __init__(self, field):
        self.field = field

    execCondition = property(lambda x: _get_(x, 'execCondition'),
                             lambda x, value: _set_(x, value, 'execCondition'))


@implementer(IFieldMetadataHandler)
class EasyFormActionMetadataHandler(object):

    """Support the easyform: namespace in model definitions.
    """

    namespace = 'http://namespaces.plone.org/supermodel/easyform'
    prefix = 'easyform'

    def read(self, fieldNode, schema, field):
        name = field.__name__
        value = fieldNode.get(ns('execCondition', self.namespace))
        data = schema.queryTaggedValue('execCondition', {})
        if value:
            data[name] = value
            schema.setTaggedValue('execCondition', data)

    def write(self, fieldNode, schema, field):
        name = field.__name__
        value = schema.queryTaggedValue('execCondition', {}).get(name, None)
        if value:
            fieldNode.set(ns('execCondition', self.namespace), value)



# -*- coding: utf-8 -*-
from .actions import IAction
from collective.easyform import easyformMessageFactory as _
from collective.easyform import config
from plone.autoform import directives

import zope.interface
import zope.schema.interfaces


MODIFY_PORTAL_CONTENT = 'cmf.ModifyPortalContent'


class ICustomScript(IAction):

    """Executes a Python script for form data"""
    directives.read_permission(ProxyRole=MODIFY_PORTAL_CONTENT)
    directives.write_permission(ProxyRole=config.EDIT_PYTHON_PERMISSION)
    ProxyRole = zope.schema.Choice(
        title=_(u'label_script_proxy', default=u'Proxy role'),
        description=_(u'help_script_proxy',
                      default=u'Role under which to run the script.'),
        default=u'none',
        required=True,
        vocabulary='easyform.ProxyRoleChoices',
    )
    directives.read_permission(ScriptBody=MODIFY_PORTAL_CONTENT)
    directives.write_permission(ScriptBody=config.EDIT_PYTHON_PERMISSION)
    ScriptBody = zope.schema.Text(
        title=_(u'label_script_body', default=u'Script body'),
        description=_(u'help_script_body', default=u'Write your script here.'),
        default=config.DEFAULT_SCRIPT,
        required=False,
        missing_value=u'',
    )



# -*- coding: utf-8 -*-
from .actions import IAction
from collective.easyform import easyformMessageFactory as _
from plone.autoform import directives
from plone.z3cform.interfaces import IFormWrapper
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope.interface import Interface

import zope.interface
import zope.schema.interfaces


class ISavedDataFormWrapper(IFormWrapper):
    pass


class IExtraData(Interface):
    dt = zope.schema.TextLine(
        title=_(u'Posting Date/Time'),
        required=False,
        default=u'',
        missing_value=u'',
    )
    HTTP_X_FORWARDED_FOR = zope.schema.TextLine(
        title=_(
            u'extra_header',
            default=u'${name} Header',
            mapping={u'name': u'HTTP_X_FORWARDED_FOR'}
        ),
        required=False,
        default=u'',
        missing_value=u'',
    )
    REMOTE_ADDR = zope.schema.TextLine(
        title=_(
            u'extra_header',
            default=u'${name} Header',
            mapping={u'name': u'REMOTE_ADDR'}
        ),
        required=False,
        default=u'',
        missing_value=u'',
    )
    HTTP_USER_AGENT = zope.schema.TextLine(
        title=_(
            u'extra_header',
            default=u'${name} Header',
            mapping={u'name': u'HTTP_USER_AGENT'}
        ),
        required=False,
        default=u'',
        missing_value=u'',
    )


class ISaveData(IAction):

    """A form action adapter that will save form input data and
       return it in csv- or tab-delimited format."""
    showFields = zope.schema.List(
        title=_(u'label_savefields_text', default=u'Saved Fields'),
        description=_(
            u'help_savefields_text',
            default=u'Pick the fields whose inputs you\'d like to include in '
                    u'the saved data. If empty, all fields will be saved.'
        ),
        unique=True,
        required=False,
        value_type=zope.schema.Choice(vocabulary='easyform.Fields'),
    )
    directives.widget(ExtraData=CheckBoxFieldWidget)
    ExtraData = zope.schema.List(
        title=_(u'label_savedataextra_text', default='Extra Data'),
        description=_(
            u'help_savedataextra_text',
            default=u'Pick any extra data you\'d like saved with the form '
                    u'input.'
        ),
        unique=True,
        value_type=zope.schema.Choice(vocabulary='easyform.ExtraDataDL'),
    )
    DownloadFormat = zope.schema.Choice(
        title=_(u'label_downloadformat_text', default=u'Download Format'),
        default=u'csv',
        vocabulary='easyform.FormatDL',
    )
    UseColumnNames = zope.schema.Bool(
        title=_(u'label_usecolumnnames_text', default=u'Include Column Names'),
        description=_(
            u'help_usecolumnnames_text',
            default=u'Do you wish to have column names on the first line of '
                    u'downloaded input?'),
        required=False,
    )
#     ExLinesField('SavedFormInput',
#     edit_accessor='getSavedFormInputForEdit',
#     mutator='setSavedFormInput',
#     searchable=0,
#     required=0,
#     primary=1,
#    schemata='saved data',
#     directives.read_permission=DOWNLOAD_SAVED_PERMISSION,
#     widget=TextAreaWidget(
#        label=_(u'label_savedatainput_text', default=u'Saved Form Input'),
#         description=_(u'help_savedatainput_text'),
#        ),
#    ),



# -*- coding: utf-8 -*-

from collective.easyform.interfaces import ILabelWidget
from collective.easyform.interfaces import IRichLabelWidget
from Products.Five.browser import BrowserView
from Products.Five.browser.metaconfigure import ViewMixinForTemplates
from z3c.form import interfaces
from z3c.form.browser import widget
from z3c.form.widget import FieldWidget
from z3c.form.widget import Widget
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.component import adapter
from zope.interface import implementer
from zope.interface import implementer_only
from zope.schema.interfaces import IField


@implementer_only(ILabelWidget)
class LabelWidget(widget.HTMLFormElement, Widget):

    """Textarea widget implementation."""

    klass = u'label-widget'
    css = u'label'
    value = u''

    def update(self):
        super(LabelWidget, self).update()
        widget.addFieldClass(self)


@adapter(IField, interfaces.IFormLayer)
@implementer(interfaces.IFieldWidget)
def LabelFieldWidget(field, request):
    """IFieldWidget factory for TextWidget."""
    return FieldWidget(field, LabelWidget(request))


@implementer_only(IRichLabelWidget)
class RichLabelWidget(widget.HTMLFormElement, Widget):

    """Textarea widget implementation."""

    klass = u'rich-label-widget'
    css = u'richlabel'
    value = u''

    def update(self):
        super(RichLabelWidget, self).update()
        widget.addFieldClass(self)


@adapter(IField, interfaces.IFormLayer)
@implementer(interfaces.IFieldWidget)
def RichLabelFieldWidget(field, request):
    """IFieldWidget factory for TextWidget."""
    return FieldWidget(field, RichLabelWidget(request))


class LabelRenderWidget(ViewMixinForTemplates, BrowserView):
    index = ViewPageTemplateFile('label.pt')


class RichLabelRenderWidget(ViewMixinForTemplates, BrowserView):
    index = ViewPageTemplateFile('rich_label.pt')



# -*- coding: utf-8 -*-
from collective.easyform import validators
from collective.easyform.api import get_schema
from collective.easyform.api import set_fields
from collective.easyform.browser.view import ValidateFile
from collective.easyform.interfaces import IFieldExtender
from collective.easyform.tests import base
from plone.namedfile.interfaces import INamed
from Products.CMFPlone.RegistrationTool import EmailAddressInvalid
from Products.validation import validation
from z3c.form.interfaces import IFormLayer
from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError
from zope.i18n import translate
from zope.interface import classImplements
from ZPublisher.BaseRequest import BaseRequest


IFieldValidator = validators.IFieldValidator

FORM_DATA = {
    'topic': u'test subject',
    'replyto': u'test@test.org',
    'comments': u'test comments',
}


class TestBaseValidators(base.EasyFormTestCase):

    """ test base validators """

    def afterSetUp(self):
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.CSRFProtection = False  # no csrf protection
        classImplements(BaseRequest, IFormLayer)
        validators.update_validators()

        request = self.layer['request']
        for i in FORM_DATA:
            request.form['form.widgets.{0}'.format(i)] = FORM_DATA[i]

    def test_defaultvalidator(self):
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(errors, ())
        self.assertEqual(data, FORM_DATA)

    def test_basevalidator(self):
        fields = get_schema(self.ff1)
        IFieldExtender(fields['replyto']).validators = ["isEmail"]
        set_fields(self.ff1, fields)
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(errors, ())
        self.assertEqual(data, FORM_DATA)

    def test_basevalidator2(self):
        fields = get_schema(self.ff1)
        IFieldExtender(fields['comments']).validators = ["isInt", "isURL"]
        set_fields(self.ff1, fields)
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(len(errors), 1)

    def test_talvalidator(self):
        fields = get_schema(self.ff1)
        IFieldExtender(
            fields['comments']).TValidator = "python: value == 'comments'"
        set_fields(self.ff1, fields)
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(errors, ())
        self.assertEqual(data, FORM_DATA)

    def test_talvalidator2(self):
        fields = get_schema(self.ff1)
        IFieldExtender(fields['comments']).TValidator = "python: !value"
        set_fields(self.ff1, fields)
        view = self.ff1.restrictedTraverse('view')
        form = view.form_instance
        form.update()

        data, errors = form.extractData()
        self.assertEqual(len(errors), 1)


class TestCustomValidators(base.EasyFormTestCase):

    """ test our validators """

    def ttest_inExNumericRange(self):
        v = validation.validatorFor('inExNumericRange')
        self.assertEqual(v(10, minval=1, maxval=20), 1)
        self.assertEqual(v('10', minval=1, maxval=20), 1)
        self.assertEqual(v('1', minval=1, maxval=20), 1)
        self.assertEqual(v('20', minval=1, maxval=20), 1)
        self.assertNotEqual(v(0, minval=1, maxval=5), 1)
        self.assertNotEqual(v(6, minval=1, maxval=5), 1)
        self.assertNotEqual(v(4, minval=5, maxval=3), 1)

    def ttest_isNotTooLong(self):
        v = validation.validatorFor('isNotTooLong')
        self.assertEqual(v('', maxlength=20), 1)
        self.assertEqual(v('1234567890', maxlength=20), 1)
        self.assertEqual(v('1234567890', maxlength=10), 1)
        self.assertEqual(v('1234567890', maxlength=0), 1)
        self.assertNotEqual(v('1234567890', maxlength=9), 1)
        self.assertNotEqual(v('1234567890', maxlength=1), 1)

    def test_isChecked(self):
        v = getUtility(IFieldValidator, name='isChecked')
        self.assertEqual(v('1'), None)
        self.assertNotEqual(v('0'), None)

    def test_isUnchecked(self):
        v = getUtility(IFieldValidator, name='isUnchecked')
        self.assertEqual(v('0'), None)
        self.assertNotEqual(v('1'), None)

    def test_isNotLinkSpam(self):
        v = getUtility(IFieldValidator, name='isNotLinkSpam')
        good = """I am link free and proud of it"""
        bad1 = """<a href="mylink">Bad.</a>"""
        bad2 = """http://bad.com"""
        bad3 = """www.Bad.com"""
        bad = (bad1, bad2, bad3)

        self.assertEqual(v(good), None)
        for b in bad:
            self.assertNotEqual(
                v(b), None, '"{0}" should be considered a link.'.format(b))

    def ttest_isNotTooLong2(self):
        v = validation.validatorFor('isNotTooLong')
        v.maxlength = 10
        self.assertEqual(v('abc'), 1)
        self.assertNotEqual(v('abcdefghijklmnopqrstuvwxyz'), 1)

        # there was a bug where widget.maxlength could possibly be defined as
        # '' which means calling int(widget.maxlength) would fail

        class Mock(object):
            pass
        field = Mock()
        field.widget = Mock()
        field.widget.maxlength = ''

        self.assertEqual(v('abc', field=field), 1)

    def test_isEmail(self):
        v = getUtility(IFieldValidator, name='isValidEmail')
        self.assertEqual(v('hi@there.com'), None)
        self.assertEqual(v('one@u.washington.edu'), None)
        self.assertRaises(EmailAddressInvalid, v, '@there.com')

    def test_isCommaSeparatedEmails(self):
        v = getUtility(IFieldValidator, name='isCommaSeparatedEmails')
        self.assertEqual(v('hi@there.com,another@two.com'), None)
        self.assertEqual(
            v('one@u.washington.edu,  two@u.washington.edu'), None)
        self.assertNotEqual(v('abc@plone.org; xyz@plone.org'), None)


class TestCustomValidatorMessages(base.EasyFormTestCase):
    """ Test friendlier validation framework """

    def test_stringValidators(self):
        """ Test string validation
        """
        validators.update_validators()

        def validator(n):
            return getUtility(IFieldValidator, name=n)

        def validate(n, v):
            return validator(n) and validator(n)(v)

        self.assertRaises(
            ComponentLookupError, validate, 'noValidator', 'test')

        self.assertNotEqual(validate('isEmail', 'test'), None)

        self.assertEqual(validate('isEmail', 'test@test.com'), None)

        self.assertEqual(validate('isZipCode', '12345'), None)
        self.assertNotEqual(validate('isZipCode', '12345-1234'), None)


class DummyFile(object):

    def __init__(self, size=1, filename=''):
        self.size = size
        self.filename = filename

    def getSize(self):
        return self.size


classImplements(DummyFile, INamed)


class TestSizeValidator(base.EasyFormTestCase):

    def afterSetUp(self):
        self.request = self.layer['request']
        self.validate_view = ValidateFile(self.portal, self.request)

    def test_filesize_none_validation(self):
        self.assertFalse(self.validate_view(None))

    def test_filesize_no_file_validation(self):
        self.assertFalse(self.validate_view('not a file'))

    def test_filiesize_smallsize_validation(self):
        self.assertFalse(self.validate_view(DummyFile(1024)))

    def test_filiesize_bigsize_validation(self):
        self.assertEqual(
            translate(self.validate_view(DummyFile(1000000000))),
            u'File is bigger than allowed size of 1048576 bytes!'
        )

    def test_filiesize_bigsize_custom_validation(self):
        self.assertEqual(
            translate(self.validate_view(DummyFile(1025), 1024)),
            u'File is bigger than allowed size of 1024 bytes!'
        )

    def test_forbidden_type_validation_fail(self):
        validation = self.validate_view(
            DummyFile(filename='foo.ZIP'), forbidden_types=('zip',))
        self.assertEqual(
            translate(validation),
            u'File type "ZIP" is not allowed!'
        )

    def test_forbidden_type_validation_pass(self):
        validation = self.validate_view(
            DummyFile(filename='foo.txt'), forbidden_types=('zip',))
        self.assertFalse(validation)

    def test_allowed_type_validation_fail(self):
        validation = self.validate_view(
            DummyFile(filename='foo.ZIP'), allowed_types=('txt',))
        self.assertEqual(
            translate(validation),
            u'File type "ZIP" is not allowed!'
        )

    def test_allowed_type_validation_pass(self):
        validation = self.validate_view(
            DummyFile(filename='foo.txt'), allowed_types=('txt',))
        self.assertFalse(validation)

    def test_allowed_type_no_ext(self):
        validation = self.validate_view(
            DummyFile(filename='foo'), allowed_types=('txt',))
        self.assertEqual(
            translate(validation),
            u'File type "" is not allowed!'
        )



# -*- coding: utf-8 -*-
#
# Integration tests for miscellaneous stuff
#

from AccessControl import Unauthorized
from collective.easyform.actions import OrderedDict
from collective.easyform.browser.fields import AjaxSaveHandler
from collective.easyform.tests import base
from plone import api
from plone.namedfile.file import NamedFile


class TestMisc(base.EasyFormTestCase):
    ''' test miscellaneous stuff '''

    def test_ordereddict_reverse(self):
        d = OrderedDict()
        d['a'] = 1
        d['b'] = 2
        d['c'] = 3
        self.assertEqual(
            d.reverse(),
            [('c', 3), ('b', 2), ('a', 1)]
        )


class TestAjaxSaveHandler(base.EasyFormTestCase):

    def test_ajax_save_handler_call_unathorized(self):
        self.folder.invokeFactory('EasyForm', 'ff1')
        view = AjaxSaveHandler(self.folder['ff1'], self.layer['request'])
        with self.assertRaises(Unauthorized):
            view()


class TestCustomTemplates(base.EasyFormTestCase):

    no_schema = u'''
  <model xmlns="http://namespaces.plone.org/supermodel/schema">
    <schema></schema>
  </model>
        '''

    def test_custom_fields(self):
        default_fields = api.content.create(
            self.portal, 'File', id='easyform_default_fields.xml')
        default_fields.file = NamedFile(self.no_schema)
        ef = api.content.create(self.folder, 'EasyForm', id='my-form')
        self.assertEqual(ef.fields_model, self.no_schema)

    def test_custom_actions(self):
        default_actions = api.content.create(
            self.portal, 'File', id='easyform_default_actions.xml')
        default_actions.file = NamedFile(self.no_schema)
        ef = api.content.create(self.folder, 'EasyForm', id='my-form')
        self.assertEqual(ef.actions_model, self.no_schema)



# -*- coding: utf-8 -*-
from email import message_from_string
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.testing import Layer
from plone.testing.z2 import ZSERVER_FIXTURE
from Products.MailHost.interfaces import IMailHost
from Products.MailHost.MailHost import MailHost
from unittest import TestCase
from zope.component import getSiteManager


class MailHostMock(MailHost):

    def _send(self, mfrom, mto, messageText, immediate=False):
        print('<sent mail from {0} to {1}>'.format(mfrom, mto))  # noqa: T003
        self.msgtext = messageText
        self.msg = message_from_string(messageText.lstrip())


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        from plone.protect import auto  # noqa
        auto.CSRF_DISABLED = True
        import collective.easyform
        self.loadZCML(package=collective.easyform)
        try:
            import plone.formwidget.recaptcha
            self.loadZCML(package=plone.formwidget.recaptcha)
        except ImportError:
            pass

    def setUpPloneSite(self, portal):
        # Install the collective.easyform product
        self.applyProfile(portal, 'collective.easyform:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        portal.manage_changeProperties(
            email_from_address='mdummy@address.com')
        portal.MailHost = mailhost = MailHostMock()
        sm = getSiteManager(context=portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)


FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name='collective.easyform:Integration',
)
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,),
    name='collective.easyform:Functional',
)
ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(FIXTURE, AUTOLOGIN_LIBRARY_FIXTURE, ZSERVER_FIXTURE),
    name='collective.easyform:Acceptance')
ROBOT_TESTING = Layer(name='collective.easyform:Robot')


class EasyFormTestCase(TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']
        self.portal.invokeFactory('Folder', 'test-folder')
        self.folder = self.portal['test-folder']
        self.afterSetUp()

    def afterSetUp(self):
        pass


class EasyFormFunctionalTestCase(TestCase):

    layer = FUNCTIONAL_TESTING



# -*- coding: utf-8 -*-
from collective.easyform import easyformMessageFactory as _  # NOQA
from collective.easyform.api import get_context
from collective.easyform.api import get_schema
from plone.schemaeditor.interfaces import IFieldFactory
from zope.component import getUtilitiesFor
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.i18nmessageid import MessageFactory
from zope.interface import provider
from zope.schema import getFieldsInOrder
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary

import operator


PMF = MessageFactory('plone')


def _make_vocabulary(items):
    return SimpleVocabulary([
        SimpleVocabulary.createTerm(token, token, name)
        for (name, token) in items
    ])


@provider(IVocabularyFactory)
def CustomActionsVocabularyFactory(context):
    items = [
        (_(u'Traverse to'), u'traverse_to'),
        (_(u'Redirect to'), u'redirect_to'),
    ]
    return _make_vocabulary(items)


@provider(IVocabularyFactory)
def MimeListVocabularyFactory(context):
    items = [
        (u'HTML', u'html'),
        (PMF(u'Text'), u'plain'),
    ]
    return _make_vocabulary(items)


@provider(IVocabularyFactory)
def FormMethodsVocabularyFactory(context):
    return SimpleVocabulary.fromValues((
        u'post',
        u'get',
    ))


@provider(IVocabularyFactory)
def XinfoHeadersVocabularyFactory(context):
    return SimpleVocabulary.fromValues((
        u'HTTP_X_FORWARDED_FOR',
        u'REMOTE_ADDR',
        u'PATH_INFO',
        u'HTTP_USER_AGENT',
        u'HTTP_REFERER',
    ))


@provider(IVocabularyFactory)
def ProxyRoleChoicesVocabularyFactory(context):
    items = [
        (u'No proxy role', u'none'),
        (u'Manager', u'Manager'),
    ]
    return _make_vocabulary(items)


@provider(IVocabularyFactory)
def ExtraDataDLVocabularyFactory(context):
    items = [
        (_(u'vocabulary_postingdt_text', default=u'Posting Date/Time'), u'dt'),
        (u'HTTP_X_FORWARDED_FOR', u'HTTP_X_FORWARDED_FOR'),
        (u'REMOTE_ADDR', u'REMOTE_ADDR'),
        (u'HTTP_USER_AGENT', u'HTTP_USER_AGENT'),
    ]
    return _make_vocabulary(items)


@provider(IVocabularyFactory)
def FormatDLVocabularyFactory(context):
    items = [
        (_(u'vocabulary_tsv_text', default=u'Tab-Separated Values'), u'tsv'),
        (_(u'vocabulary_csv_text', default=u'Comma-Separated Values'), u'csv'),
    ]
    return _make_vocabulary(items)


@provider(IVocabularyFactory)
def FieldsVocabularyFactory(context):
    terms = []
    if hasattr(context, 'interface'):
        form = get_context(context)
    elif hasattr(context, 'fields_model'):
        form = context
    else:
        return SimpleVocabulary(terms)
    fields = getFieldsInOrder(get_schema(form))
    for name, field in fields:
        terms.append(
            SimpleVocabulary.createTerm(
                name,
                str(name),
                field.title
            )
        )
    return SimpleVocabulary(terms)


@provider(IVocabularyFactory)
def EasyFormActionsVocabularyFactory(context):
    """EasyForm actions vocabulary"""
    from collective.easyform.interfaces import IActionFactory
    return SimpleVocabulary([
        SimpleVocabulary.createTerm(
            factory, translate(factory.title), factory.title)
        for (id, factory) in getUtilitiesFor(IActionFactory)
        if factory.available(context)
    ])


@provider(IVocabularyFactory)
def ValidatorsVocabularyFactory(context):
    """Field validators vocabulary"""
    from collective.easyform.interfaces import IFieldValidator
    return SimpleVocabulary([
        SimpleVocabulary.createTerm(i, i, i)
        for i, u in getUtilitiesFor(IFieldValidator)
    ])


@provider(IVocabularyFactory)
def SchemaEditorFieldsVocabularyFactory(context=None):
    request = getRequest()
    terms = []
    for (field_id, factory) in getUtilitiesFor(IFieldFactory):
        terms.append(
            SimpleVocabulary.createTerm(
                field_id,
                factory.title,
                translate(factory.title, context=request)
            )
        )
    terms = sorted(terms, key=operator.attrgetter('title'))
    return SimpleVocabulary(terms)



# -*- coding: utf-8 -*-
#
# Integration tests specific to save-data adapter.
#

from collective.easyform.api import get_actions
from collective.easyform.api import get_schema
from collective.easyform.interfaces import ISaveData
from collective.easyform.tests import base
from StringIO import StringIO
from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPResponse import HTTPResponse

import plone.protect
import sys


def FakeRequest(method='GET', add_auth=False, **kwargs):
    environ = {}
    environ.setdefault('SERVER_NAME', 'foo')
    environ.setdefault('SERVER_PORT', '80')
    environ.setdefault('REQUEST_METHOD', method)
    request = HTTPRequest(sys.stdin,
                          environ,
                          HTTPResponse(stdout=StringIO()))
    request.form = kwargs
    if add_auth:
        request.form['_authenticator'] = plone.protect.createToken()
    return request


class TestFunctions(base.EasyFormTestCase):

    ''' test save data adapter '''

    def afterSetUp(self):
        base.EasyFormTestCase.afterSetUp(self)
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')

    def createSaver(self):
        ''' Creates FormCustomScript object '''
        # 1. Create custom script adapter in the form folder
        self.portal.REQUEST['form.widgets.title'] = u'Saver'
        self.portal.REQUEST['form.widgets.__name__'] = u'saver'
        self.portal.REQUEST['form.widgets.description'] = u''
        self.portal.REQUEST['form.widgets.factory'] = ['Save Data']
        self.portal.REQUEST['form.buttons.add'] = u'Add'
        view = self.ff1.restrictedTraverse('actions/@@add-action')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)

        # 2. Check that creation succeeded
        actions = get_actions(self.ff1)
        self.assertTrue('saver' in actions)

    def testSavedDataView(self):
        ''' test saved data view '''

        self.createSaver()

        view = self.ff1.restrictedTraverse('saveddata')
        self.assertEqual(view.items(), [('saver', u'Saver')])

    def testSaverDataFormExtraData(self):
        ''' test saver data form extra data'''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.itemsSaved(), 0)
        request = FakeRequest(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.ExtraData = ('dt',)
        saver.onSuccess(request.form, request)

        view = self.ff1.restrictedTraverse('@@actions')
        view = view.publishTraverse(view.request, 'saver')
        view = view.publishTraverse(view.request, 'data')
        view.update()
        form = view.form_instance
        message = form.description()
        self.assertEqual(message.mapping, {'items': 1})
        item = form.get_items()[0]
        self.assertEqual(item[1]['id'], item[0])
        self.assertEqual(item[1]['topic'], 'test subject')
        self.assertEqual(item[1]['replyto'], 'test@test.org')
        self.assertEqual(item[1]['comments'], 'test comments')
        self.assertTrue(' ' in item[1]['dt'])

    def testSaverDataFormShowFields(self):
        ''' test saver data form show fields '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.itemsSaved(), 0)
        request = FakeRequest(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.showFields = ('topic', 'comments')
        saver.onSuccess(request.form, request)

        view = self.ff1.restrictedTraverse('@@actions')
        view = view.publishTraverse(view.request, 'saver')
        view = view.publishTraverse(view.request, 'data')
        view.update()
        form = view.form_instance
        message = form.description()
        self.assertEqual(message.mapping, {'items': 1})
        item = form.get_items()[0]
        self.assertEqual(item[1]['id'], item[0])
        self.assertEqual(item[1]['topic'], 'test subject')
        self.assertEqual(item[1]['comments'], 'test comments')
        self.assertTrue('replyto' not in item[1])

    def testSaverDataFormOneItem(self):
        ''' test saver data form one item '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.itemsSaved(), 0)
        request = FakeRequest(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.onSuccess(request.form, request)

        view = self.ff1.restrictedTraverse('@@actions')
        view = view.publishTraverse(view.request, 'saver')
        view = view.publishTraverse(view.request, 'data')
        view.update()
        form = view.form_instance
        message = form.description()
        self.assertEqual(message.mapping, {'items': 1})
        item = form.get_items()[0]
        self.assertEqual(item[1]['id'], item[0])
        self.assertEqual(item[1]['topic'], 'test subject')
        self.assertEqual(item[1]['replyto'], 'test@test.org')
        self.assertEqual(item[1]['comments'], 'test comments')

    def testSaverDataForm(self):
        ''' test saver data form '''

        self.createSaver()

        view = self.ff1.restrictedTraverse('@@actions')
        view = view.publishTraverse(view.request, 'saver')
        view = view.publishTraverse(view.request, 'data')
        view.update()
        form = view.form_instance
        message = form.description()
        self.assertEqual(message.mapping, {'items': 0})
        self.assertEqual([i for i in form.get_items()], [])

    def testSaver(self):
        ''' test save data adapter action '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']

        self.assertEqual(saver.itemsSaved(), 0)

        res = saver.getSavedFormInputForEdit()
        self.assertEqual(res, '')

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.onSuccess(request.form, request)

        self.assertEqual(saver.itemsSaved(), 1)

        res = saver.getSavedFormInputForEdit()
        self.assertEqual(
            res.strip(), 'test@test.org,test subject,test comments')

    def testSaverExtraData(self):
        ''' test save data adapter action '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.itemsSaved(), 0)

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.ExtraData = ('dt',)
        saver.onSuccess(request.form, request)

        self.assertEqual(saver.itemsSaved(), 1)
        self.assertTrue('dt' in saver.getSavedFormInput()[0])

    def testSaverDownload(self):
        ''' test save data '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.itemsSaved(), 0)

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.onSuccess(request.form, request)

        self.assertEqual(saver.itemsSaved(), 1)
        saver.download(request.response)
        res = request.response.stdout.getvalue()
        self.assertTrue('Content-Type: text/comma-separated-values' in res)
        self.assertTrue(
            'Content-Disposition: attachment; filename="saver.csv"' in res)
        self.assertTrue(saver.getSavedFormInputForEdit() in res)

    def testSaverDownloadTSV(self):
        ''' test save data '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.itemsSaved(), 0)

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.onSuccess(request.form, request)

        self.assertEqual(saver.itemsSaved(), 1)
        saver.DownloadFormat = 'tsv'
        saver.download(request.response)
        res = request.response.stdout.getvalue()
        self.assertTrue('Content-Type: text/tab-separated-values' in res)
        self.assertTrue(
            'Content-Disposition: attachment; filename="saver.tsv"' in res)
        self.assertTrue(saver.getSavedFormInputForEdit(delimiter='\t') in res)

    def testSaverDownloadWithTitles(self):
        ''' test save data '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.itemsSaved(), 0)

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.onSuccess(request.form, request)

        self.assertEqual(saver.itemsSaved(), 1)
        saver.UseColumnNames = True
        saver.download(request.response)
        res = request.response.stdout.getvalue()
        self.assertTrue('Content-Type: text/comma-separated-values' in res)
        self.assertTrue(
            'Content-Disposition: attachment; filename="saver.csv"' in res)
        self.assertTrue(saver.getSavedFormInputForEdit(header=True) in res)

    def testSaverDownloadExtraData(self):
        ''' test save data '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.itemsSaved(), 0)

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.ExtraData = ('dt', 'HTTP_USER_AGENT')
        saver.onSuccess(request.form, request)

        self.assertEqual(saver.itemsSaved(), 1)
        saver.download(request.response)
        res = request.response.stdout.getvalue()
        self.assertTrue('Content-Type: text/comma-separated-values' in res)
        self.assertTrue(
            'Content-Disposition: attachment; filename="saver.csv"' in res)
        self.assertTrue(saver.getSavedFormInputForEdit() in res)

    def testSaverSavedFormInput(self):
        """ test save data adapter action and direct access to
        SavedFormInput """

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.onSuccess(request.form, request)

        self.assertEqual(saver.itemsSaved(), 1)
        row = iter(saver.getSavedFormInput()).next()
        self.assertEqual(len(row), 4)

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.onSuccess(request.form, request)
        self.assertEqual(saver.itemsSaved(), 2)

        saver.clearSavedFormInput()
        self.assertEqual(saver.itemsSaved(), 0)

    def testSetSavedFormInput(self):
        ''' test setSavedFormInput functionality '''

        # set up saver
        self.createSaver()
        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        self.assertEqual(saver.getSavedFormInputForEdit(), '')

        # save a row
        fields = list(get_schema(self.ff1))
        saver.addDataRow(dict(zip(fields, ['one', 'two', 'three'])))
        self.assertEqual(saver.itemsSaved(), 1)
        items = saver.getSavedFormInputItems()
        self.assertEqual(
            items[0][1],
            dict(zip(['id'] + fields, [items[0][0], 'one', 'two', 'three']))
        )
        self.assertEqual(saver.getSavedFormInputForEdit(), 'one,two,three\r\n')

        # save a couple of \n-delimited rows - \n eol
        saver.addDataRow(dict(zip(fields, ['four', 'five', 'six'])))
        self.assertEqual(saver.itemsSaved(), 2)
        items = saver.getSavedFormInputItems()
        self.assertEqual(
            items[0][1],
            dict(zip(['id'] + fields, [items[0][0], 'one', 'two', 'three']))
        )
        self.assertEqual(
            items[1][1],
            dict(zip(['id'] + fields, [items[1][0], 'four', 'five', 'six']))
        )
        self.assertEqual(
            saver.getSavedFormInputForEdit(),
            'one,two,three\r\nfour,five,six\r\n'
        )

        # save empty string
        saver.clearSavedFormInput()
        self.assertEqual(saver.itemsSaved(), 0)
        self.assertEqual(saver.getSavedFormInputForEdit(), '')

    def testEditSavedFormInput(self):
        ''' test manage_saveData functionality '''

        # set up saver
        self.createSaver()
        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']

        # save a row
        fields = list(get_schema(self.ff1))
        # saver.savedFormInput = 'one,two,three'
        saver.addDataRow(dict(zip(fields, ['one', 'two', 'three'])))
        self.assertEqual(saver.itemsSaved(), 1)
        items = saver.getSavedFormInputItems()
        self.assertEqual(
            items[0][1],
            dict(zip(['id'] + fields, [items[0][0], 'one', 'two', 'three']))
        )

    def testDeleteSavedFormInput(self):
        ''' test manage_deleteData functionality '''

        # set up saver
        self.createSaver()
        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']

        # save a few rows
        fields = list(get_schema(self.ff1))
        saver.addDataRow(dict(zip(fields, ['one', 'two', 'three'])))
        saver.addDataRow(dict(zip(fields, ['four', 'five', 'six'])))
        saver.addDataRow(dict(zip(fields, ['seven', 'eight', 'nine'])))
        self.assertEqual(saver.itemsSaved(), 3)

        # saver.manage_deleteData(saver._storage.keys()[1])
        saver.delDataRow(saver.getSavedFormInputItems()[1][0])
        self.assertEqual(saver.itemsSaved(), 2)
        items = saver.getSavedFormInputItems()
        self.assertEqual(
            items[0][1],
            dict(zip(['id'] + fields, [items[0][0], 'one', 'two', 'three']))
        )
        self.assertEqual(
            items[1][1],
            dict(zip(['id'] + fields, [items[1][0], 'seven', 'eight', 'nine']))
        )

    def testSaverInputAsDictionaries(self):
        ''' test save data adapter's InputAsDictionaries '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']

        # self.ff1.setActionAdapter(('saver',))

        # self.assertEqual(saver.inputAsDictionaries,
        # saver.InputAsDictionaries)

        self.assertEqual(saver.itemsSaved(), 0)

        request = FakeRequest(
            add_auth=True, method='POST', topic='test subject',
            replyto='test@test.org',
            comments='test comments')
        saver.onSuccess(request.form, request)
        # errors = self.ff1.fgvalidate(REQUEST=request)
        # self.assertEqual(errors, {})

        self.assertEqual(saver.itemsSaved(), 1)

        iad = saver.getSavedFormInput()
        row = iter(iad).next()
        self.assertEqual(len(row), 4)
        self.assertEqual(row['topic'], 'test subject')

    def testSaverColumnNames(self):
        ''' test save data adapter's getColumnNames function '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']

        cn = saver.getColumnNames()
        self.assertTrue(len(cn) == 3)
        self.assertTrue(cn[0] == 'replyto')
        self.assertTrue(cn[1] == 'topic')
        self.assertTrue(cn[2] == 'comments')

        # Use selective field saving
        saver.showFields = ('topic', 'comments')
        cn = saver.getColumnNames()
        self.assertTrue(len(cn) == 2)
        self.assertTrue(cn[0] == 'topic')
        self.assertTrue(cn[1] == 'comments')
        saver.showFields = tuple()

        # Add an extra column
        saver.ExtraData = ('dt',)
        cn = saver.getColumnNames()
        self.assertTrue(len(cn) == 4)
        self.assertTrue(cn[3] == 'dt')

    def testSaverColumnTitles(self):
        ''' test save data adapter's getColumnTitles function '''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']

        cn = saver.getColumnTitles()
        self.assertTrue(len(cn) == 3)
        self.assertTrue(cn[0] == 'Your E-Mail Address')
        self.assertTrue(cn[1] == 'Subject')
        self.assertTrue(cn[2] == 'Comments')

        # Add an extra column
        saver.ExtraData = ('dt',)
        cn = saver.getColumnTitles()
        self.assertTrue(len(cn) == 4)
        self.assertTrue(cn[3] == 'Posting Date/Time')

    def testSaverSelectiveFieldSaving(self):
        ''' Test selective inclusion of fields in the data'''

        self.createSaver()

        self.assertTrue('saver' in get_actions(self.ff1))
        saver = get_actions(self.ff1)['saver']
        saver.showFields = ('topic', 'comments')

        request = FakeRequest(
            add_auth=True,
            method='POST',
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )
        saver.onSuccess(request.form, request)

        self.assertEqual(saver.itemsSaved(), 1)
        row = iter(saver.getSavedFormInput()).next()
        self.assertEqual(len(row), 3)
        self.assertEqual(row['topic'], 'test subject')
        self.assertEqual(row['comments'], 'test comments')

    def testGetSaveDataAdaptersView(self):
        ''' test the @@get_save_data_adapters view '''

        self.createSaver()

        view = self.ff1.restrictedTraverse('@@get_save_data_adapters')
        results = view()
        self.assertEqual(len(results), 1)
        saver = results[0]
        self.assertTrue(ISaveData.providedBy(saver))



# -*- coding: utf-8 -*-
from collective.easyform import easyformMessageFactory as _
from collective.easyform.interfaces import IFieldValidator
from plone import api
from Products.CMFPlone.RegistrationTool import EmailAddressInvalid
from Products.validation import validation
from Products.validation.validators.BaseValidators import baseValidators
from zope.component import provideUtility


BAD_SIGNS = frozenset(['<a ', 'www.', 'http:', '.com', 'https:'])


def isValidEmail(value):
    """Check for the user email address"""
    reg_tool = api.portal.get_tool('portal_registration')
    if not (value and reg_tool.isValidEmail(value)):
        raise EmailAddressInvalid


def isCommaSeparatedEmails(value):
    """Check for one or more E-Mail Addresses separated by commas"""
    reg_tool = api.portal.get_tool('portal_registration')
    for v in value.split(','):
        if not reg_tool.isValidEmail(v.strip()):
            return _(
                u'Must be a valid list of email addresses '
                u'(separated by commas).'
            )


def isChecked(value):
    if not (
        (isinstance(value, bool) and value) or
        (isinstance(value, basestring) and value == '1')
    ):
        return _(u'Must be checked.')


def isUnchecked(value):
    if not isChecked(value):
        return _(u'Must be unchecked.')


def isNotLinkSpam(value):
    if not value:
        return    # No value can't be SPAM
    # validation is optional and configured on the field
    value = value.lower()
    for s in BAD_SIGNS:
        if s in value:
            return _('Links are not allowed.')


# Base validators
def update_validators():
    if validation and baseValidators:
        def method(name):
            def validate(value):
                if isinstance(value, unicode):
                    value = value.encode('utf-8')
                res = validation(name, value)
                if res != 1:
                    return res
            return validate
        for validator in baseValidators:
            provideUtility(
                method(validator.name),
                provides=IFieldValidator,
                name=validator.name
            )


update_validators()



# -*- coding: utf-8 -*-
from .validators import isTALES
from collective.easyform import easyformMessageFactory as _  # NOQA
from collective.easyform import config
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.schemaeditor.interfaces import IFieldContext
from plone.schemaeditor.interfaces import IFieldEditorExtender
from plone.schemaeditor.interfaces import ISchemaContext
from plone.supermodel.model import fieldset
from plone.supermodel.model import Schema
from z3c.form.interfaces import IFieldWidget
from zope.component import getGlobalSiteManager
from zope.interface import Interface
from zope.interface import provider
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary

import z3c.form.interfaces
import zope.interface
import zope.schema.interfaces


class WidgetVocabulary(SimpleVocabulary):

    def getTerm(self, value):
        """See zope.schema.interfaces.IBaseVocabulary"""
        if not isinstance(value, basestring):
            value = '{0}.{1}'.format(
                value.widget_factory.__module__, value.widget_factory.__name__)
        return self.getTermByToken(value)


@provider(IContextSourceBinder)
def widgetsFactory(context):
    terms = []
    adapters = [
        a.factory
        for a in getGlobalSiteManager().registeredAdapters()
        if (
            a.provided == IFieldWidget and
            len(a.required) == 2 and
            a.required[0].providedBy(context)
        )
    ]
    for adapter in set(adapters):
        name = u'{0}.{1}'.format(
            adapter.__module__, adapter.__name__)
        terms.append(WidgetVocabulary.createTerm(
            name, str(name), adapter.__name__))
    return WidgetVocabulary(terms)


class IEasyFormFieldsEditorExtender(IFieldEditorExtender):
    pass


class IFieldExtender(Schema):
    field_widget = zope.schema.Choice(
        title=_(u'label_field_widget',
                default=u'Field Widget'),
        description=_(u'help_field_widget', default=u''),
        required=False,
        source=widgetsFactory,
    )
    directives.write_permission(THidden=config.EDIT_TALES_PERMISSION)
    THidden = zope.schema.Bool(
        title=_(u'label_hidden',
                default=u'Hidden'),
        description=_(u'help_hidden', default=u'Field is hidden'),
        required=False,
        default=False,
    )
    fieldset(u'overrides', label=_('Overrides'),
             fields=['TDefault', 'TEnabled', 'TValidator', 'serverSide'])
    directives.write_permission(TDefault=config.EDIT_TALES_PERMISSION)
    TDefault = zope.schema.TextLine(
        title=_(u'label_tdefault_text', default=u'Default Expression'),
        description=_(
            u'help_tdefault_text',
            default=u'A TALES expression that will be evaluated when the form'
                    u'is displayed to get the field default value. Leave '
                    u'empty if unneeded. Your expression should evaluate as a '
                    u'string. PLEASE NOTE: errors in the evaluation of this '
                    u'expression will cause an error on form display.'
        ),
        default=u'',
        constraint=isTALES,
        required=False,
    )
    directives.write_permission(TEnabled=config.EDIT_TALES_PERMISSION)
    TEnabled = zope.schema.TextLine(
        title=_(u'label_tenabled_text', default=u'Enabling Expression'),
        description=_(
            u'help_tenabled_text',
            default=u'A TALES expression that will be evaluated when the form '
                    u'is displayed to determine whether or not the field is '
                    u'enabled. Your expression should evaluate as True if '
                    u'the field should be included in the form, False if it '
                    u'should be omitted. Leave this expression field empty '
                    u'if unneeded: the field will be included. PLEASE NOTE: '
                    u'errors in the evaluation of this expression will cause '
                    u'an error on form display.'
        ),
        default=u'',
        constraint=isTALES,
        required=False,
    )
    directives.write_permission(TValidator=config.EDIT_TALES_PERMISSION)
    TValidator = zope.schema.TextLine(
        title=_(u'label_tvalidator_text', default=u'Custom Validator'),
        description=_(
            u'help_tvalidator_text',
            default=u'A TALES expression that will be evaluated when the form '
                    u'is validated. Validate against \'value\', which will '
                    u'contain the field input. Return False if valid; if not '
                    u'valid return a string error message. E.G., '
                    u'"python: test(value==\'eggs\', False, \'input must be '
                    u'eggs\')" will require "eggs" for input. '
                    u'PLEASE NOTE: errors in the evaluation of this '
                    u'expression will cause an error on form display.'
        ),
        default=u'',
        constraint=isTALES,
        required=False,
    )
    directives.write_permission(serverSide=config.EDIT_TALES_PERMISSION)
    serverSide = zope.schema.Bool(
        title=_(u'label_server_side_text', default=u'Server-Side Variable'),
        description=_(u'description_server_side_text', default=u''
                      u'Mark this field as a value to be injected into the '
                      u'request form for use by action adapters and is not '
                      u'modifiable by or exposed to the client.'),
        default=False,
        required=False,
    )
    validators = zope.schema.List(
        title=_('Validators'),
        description=_(
            u'help_userfield_validators',
            default=u'Select the validators to use on this field'),
        unique=True,
        required=False,
        value_type=zope.schema.Choice(
            vocabulary='easyform.Validators'),
    )


class IEasyFormFieldsContext(ISchemaContext):

    """
    EasyForm schema view interface
    """


class IEasyFormFieldContext(IFieldContext):

    """
    EasyForm field content marker
    """


class ILabel(zope.schema.interfaces.IField):

    """Label Field."""


class IRichLabel(ILabel):

    """Rich Label Field."""
    rich_label = RichText(
        title=_(u'Rich Label'),
        default=u'',
        missing_value=u'',
    )


class ILabelWidget(z3c.form.interfaces.IWidget):

    """Label Widget."""


class IRichLabelWidget(ILabelWidget):

    """Rich Label Field Widget."""


class IReCaptcha(zope.schema.interfaces.ITextLine):

    """ReCaptcha Field."""


class IFieldValidator(Interface):

    """Base marker for field validators"""



# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            'collective.easyform:uninstall',
            'collective.easyform:testing',
        ]



# -*- coding: utf-8 -*-
from collective.easyform.api import get_actions
from collective.easyform.api import set_actions
from collective.easyform.tests import base
from z3c.form.interfaces import IFormLayer
from zope.interface import classImplements
from ZPublisher.BaseRequest import BaseRequest

import transaction


class FakeRequest(dict):

    def __init__(self, **kwargs):
        self.form = kwargs


class TrueOnceCalled(object):

    """ A mock function that evaluates to True once it has been called. """

    def __init__(self):
        self.called = False

    def __call__(self, *args, **kw):
        self.called = True

    def __bool__(self):
        return self.called


class TestEmbedding(base.EasyFormTestCase):

    """ test embedding of a PFG in another template """

    def dummy_send(self, mfrom, mto, messageText, immediate=False):
        self.mfrom = mfrom
        self.mto = mto
        self.messageText = messageText

    def LoadRequestForm(self, **kwargs):
        form = self.app.REQUEST.form
        form.clear()
        for key in kwargs.keys():
            form[key] = kwargs[key]
        return self.app.REQUEST

    def afterSetUp(self):
        base.EasyFormTestCase.afterSetUp(self)
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.title = u'ff1'
        self.ff1.CSRFProtection = False  # no csrf protection
        actions = get_actions(self.ff1)
        actions['mailer'].recipient_email = u'mdummy@address.com'
        set_actions(self.ff1, actions)
        self.mailhost = self.folder.MailHost
        self.mailhost._send = self.dummy_send
        classImplements(BaseRequest, IFormLayer)

    def test_embedded_form_renders(self):
        view = self.ff1.restrictedTraverse('@@embedded')
        res = view()

        # form renders
        self.assertTrue('Your E-Mail Address' in res)

        # form action equals request URL
        self.assertTrue(
            'action="{url}"'.format(url=self.ff1.absolute_url()) in res)

        # no form prefix
        # self.assertTrue('name="form.submitted"' in res)

        # we can specify a form prefix
        view.prefix = 'mypfg'
        res = view()
        self.assertTrue('name="mypfg.buttons.submit"' in res)

    def test_embedded_form_validates(self):
        # fake an incomplete form submission
        self.LoadRequestForm(**{
            'mypfg.buttons.submit': u'Submit',
        })

        # render the form
        view = self.ff1.restrictedTraverse('@@embedded')
        view.prefix = 'mypfg'
        res = view()

        # should stay on same page on errors, and show messages
        self.assertTrue('Required input is missing.' in res)

    def test_doesnt_process_submission_of_other_form(self):
        # fake submission of a *different* form (note mismatch of form
        # submission marker with prefix)
        self.LoadRequestForm(**{
            'form.buttons.submit': u'Submit',
        })

        # let's preset a faux controller_state (as if from the other form)
        # to make sure it doesn't throw things off
        # self.app.REQUEST.set('controller_state', 'foobar')

        # render the form
        view = self.ff1.restrictedTraverse('@@embedded')
        view.prefix = 'mypfg'
        res = view()

        # should be no validation errors
        self.assertFalse('Required input is missing.' in res)

        # (and request should still have the 'form.submitted' key)
        # self.assertTrue('form.submitted' in self.app.REQUEST.form)

        # (and the controller state should be untouched)
        # self.assertEqual(self.app.REQUEST.get('controller_state'), 'foobar')

        # but if we remove the form prefix then it should process the form
        view.prefix = 'form'
        res = view()
        self.assertTrue('Required input is missing.' in res)

    def test_render_thank_you_on_success(self):
        # We need to be able to make sure the transaction commit was called
        # before the Retry exception, without actually committing our test
        # fixtures.
        real_transaction_commit = transaction.commit
        transaction.commit = committed = TrueOnceCalled()

        self.LoadRequestForm(**{
            'form.widgets.topic': u'monkeys',
            'form.widgets.comments': u'I am not a walnut.',
            'form.widgets.replyto': u'foobar@example.com',
            'form.buttons.submit': u'Submit',
        })
        # should raise a retry exception triggering a new publish attempt
        # with the new URL
        # XXX do a full publish for this test
        self.app.REQUEST._orig_env['PATH_TRANSLATED'] = '/plone'
        self.app.REQUEST.method = 'POST'
        view = self.ff1.restrictedTraverse('@@embedded')
        # self.assertRaises(Retry, view)
        res = view()

        self.assertTrue('Thanks for your input.' in res)

        # make sure the transaction was committed
        self.assertTrue(committed)

        # make sure it can deal with VHM URLs
        self.app.REQUEST._orig_env[
            'PATH_TRANSLATED'
        ] = '/VirtualHostBase/http/nohost:80/VirtualHostRoot'
        view = self.ff1.restrictedTraverse('@@embedded')
        res = view()

        self.assertTrue('Thanks for your input.' in res)
        # clean up
        transaction.commit = real_transaction_commit



# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import safe_unicode

import os


this_path = os.path.dirname(__file__)


EDIT_TALES_PERMISSION = 'collective.easyform.EditTALESFields'
EDIT_PYTHON_PERMISSION = 'collective.easyform.EditPythonFields'
EDIT_ADVANCED_PERMISSION = 'collective.easyform.EditAdvancedFields'
EDIT_ADDRESSING_PERMISSION = 'collective.easyform.EditMailAddresses'
USE_ENCRYPTION_PERMISSION = 'collective.easyform.EditEncryptionSpecs'
DOWNLOAD_SAVED_PERMISSION = 'collective.easyform.DownloadSavedInput'

MODEL_DEFAULT = safe_unicode(open(os.path.join(
    this_path, "default_schemata", "model_default.xml")).read())

FIELDS_DEFAULT = safe_unicode(open(os.path.join(
    this_path, "default_schemata", "fields_default.xml")).read())

ACTIONS_DEFAULT = safe_unicode(open(os.path.join(
    this_path, "default_schemata", "actions_default.xml")).read())

MAIL_BODY_DEFAULT = safe_unicode(open(os.path.join(
    this_path, "default_schemata", "mail_body_default.pt")).read())


DEFAULT_SCRIPT = u"""
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=
##

# Available parameters:
#  fields  = HTTP request form fields as key value pairs
#  request = The current HTTP request.
#            Access fields by request.form["myfieldname"]
#  easyform = EasyForm object
#
# Return value is not processed -- unless you
# return a dictionary with contents. That's regarded
# as an error and will stop processing of actions
# and return the user to the form. Error dictionaries
# should be of the form {'field_id':'Error message'}


assert False, "Please complete your script"

"""



# -*- coding: utf-8 -*-
'''

    Unit test for EasyForm custom scripts

    Copyright 2006 Red Innovation http://www.redinnovation.com

'''
from AccessControl import ClassSecurityInfo
from AccessControl import Unauthorized
from App.class_init import InitializeClass
from collective.easyform.api import get_actions
from collective.easyform.tests import base
from plone.app.testing import logout
from Products.CMFCore import permissions


test_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=Succesfully working script
##

from Products.CMFCore.utils import getToolByName

assert fields['test_field'] == '123'
return 'foo'
'''

bad_parameters_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=
##title=
##

return 'foo'
'''

syntax_error_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=
##
if:
return asdfaf
'''

runtime_error_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform
##title=
##
return 'asdfaf' + 1
'''

security_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=Script needing special privileges
##

from Products.CMFCore.utils import getToolByName

portal_url = getToolByName(context, 'portal_url')
portal = portal_url.getPortalObject()

print portal

# Try set left_slots
portal.manage_addProperty('foo', ['foo'], 'lines')
'''

proxied_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=Script needing special privileges
##

# Should raise Unauthorized
return request.fooProtected()
'''

return_error_script = '''
## Python Script
##bind container=container
##bind context=context
##bind subpath=traverse_subpath
##parameters=fields, easyform, request
##title=
##
return {'comments': 'Please enter more text'}
'''

default_params_script = '''
fields
easyform
request
'''


class FakeRequest(dict):

    def __init__(self, **kwargs):
        self.form = kwargs


class SecureFakeRequest(dict):

    security = ClassSecurityInfo()

    def __init__(self, **kwargs):
        self.form = kwargs

    @security.protected(permissions.ManagePortal)
    def fooProtected(self):
        ''' Only manager can access this '''
        return 'foo'


InitializeClass(SecureFakeRequest)


class TestCustomScript(base.EasyFormTestCase):

    ''' Test FormCustomScriptAdapter functionality in EasyForm '''

    def afterSetUp(self):
        self.request = self.layer['request']

        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.CSRFProtection = False
        self.request['form.widgets.title'] = u'Test field'
        self.request['form.widgets.__name__'] = u'test_field'
        self.request['form.widgets.description'] = u'foobar'
        self.request['form.widgets.factory'] = ['label_textline_field']
        self.request['form.widgets.required'] = ['selected']
        self.request['form.buttons.add'] = u'Add'
        view = self.ff1.restrictedTraverse('fields/@@add-field')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)

    def createScript(self):
        ''' Creates FormCustomScript object '''
        # 1. Create custom script adapter in the form folder
        self.request['form.widgets.title'] = u'Adapter'
        self.request['form.widgets.__name__'] = u'adapter'
        self.request['form.widgets.description'] = u''
        self.request['form.widgets.factory'] = ['Custom Script']
        self.request['form.buttons.add'] = u'Add'
        view = self.ff1.restrictedTraverse('actions/@@add-action')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)
        # 2. Check that creation succeeded
        actions = get_actions(self.ff1)
        self.assertTrue('adapter' in actions)

    def testSuccess(self):
        ''' Succesful script execution

        Creates a script, some form content,
        executes form handling.
        '''

        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = test_script

        req = FakeRequest(test_field='123')

        reply = adapter.onSuccess({}, req)
        assert reply == 'foo', 'Script returned:' + str(reply)

    def testRunTimeError(self):
        ''' Script has run-time error '''
        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = runtime_error_script

        # Execute script
        throwed = False
        try:
            reply = adapter.onSuccess({})
        except TypeError:
            reply = None
            throwed = True

        assert throwed, "Bad script didn't throw run-time exception, got " + \
            str(reply)
        assert reply is None

    def testSyntaxError(self):
        ''' Script has syntax errors

        TODO: Syntax errors are not returned in validation?
        '''

        # Note: this test logs an error message; it does not indicate test
        # failure

        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = syntax_error_script

        # Execute script
        throwed = False
        try:
            adapter.onSuccess({}, FakeRequest())
        except ValueError:
            throwed = True

        assert throwed, "Bad script didn't throw run-time exception"

    def testBadParameters(self):
        ''' Invalid number of script parameters '''

        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = bad_parameters_script

        # Execute script
        throwed = False
        try:
            adapter.onSuccess([])
        except TypeError:
            throwed = True
        assert throwed, 'Invalid parameters failed silently'

    def testDefaultParameters(self):
        ''' Test to make sure the documented parameters are available '''

        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = default_params_script

        request = FakeRequest(
            topic='test subject',
            replyto='test@test.org',
            comments='test comments'
        )

        errors = adapter.onSuccess({}, request)
        self.assertEqual(errors, None)

    def testSecurity(self):
        ''' Script needing proxy role

        TODO: Why no security exceptions are raised?
        '''
        self.createScript()
        logout()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = security_script

        # Execute script
        throwed = False
        try:
            adapter.onSuccess({}, FakeRequest())
        except Unauthorized:
            throwed = True

        if self.portal.hasProperty('foo'):
            assert 'Script executed under full priviledges'

        self.assertTrue(throwed, 'Bypassed security, baaad!')

        adapter.ProxyRole = u'Manager'
        throwed = False
        try:
            adapter.onSuccess({}, FakeRequest())
        except Unauthorized:
            throwed = True

        if not self.portal.hasProperty('foo'):
            assert 'Script not executed thru proxy role'
        self.assertFalse(throwed, 'Unauthorized was raised!')

    def testSetProxyRole(self):
        ''' Exercise setProxyRole '''
        self.createScript()
        self.request['form.widgets.title'] = u'Adapter'
        self.request['form.widgets.description'] = u''
        self.request['form.widgets.ProxyRole'] = [u'Manager']
        self.request['form.widgets.ScriptBody'] = unicode(proxied_script)
        self.request['form.widgets.IActionExtender.execCondition'] = u''
        self.request['form.buttons.save'] = u'Save'
        view = self.ff1.restrictedTraverse('actions')
        view = view.publishTraverse(view.request, 'adapter')
        view = view.publishTraverse(view.request, 'adapter')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)
        self.request['form.widgets.title'] = u'Adapter'
        self.request['form.widgets.description'] = u''
        self.request['form.widgets.ProxyRole'] = [u'none']
        self.request['form.widgets.ScriptBody'] = unicode(proxied_script)
        self.request['form.widgets.IActionExtender.execCondition'] = u''
        self.request['form.buttons.save'] = u'Save'
        view = self.ff1.restrictedTraverse('actions')
        view = view.publishTraverse(view.request, 'adapter')
        view = view.publishTraverse(view.request, 'adapter')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)
        self.request['form.widgets.title'] = u'Adapter'
        self.request['form.widgets.description'] = u''
        self.request['form.widgets.ProxyRole'] = [u'bogus']
        self.request['form.widgets.ScriptBody'] = unicode(proxied_script)
        self.request['form.widgets.IActionExtender.execCondition'] = u''
        self.request['form.buttons.save'] = u'Save'
        view = self.ff1.restrictedTraverse('actions')
        view = view.publishTraverse(view.request, 'adapter')
        view = view.publishTraverse(view.request, 'adapter')
        view.update()
        form = view.form_instance
        data, errors = form.extractData()
        self.assertEqual(len(errors), 1)
        self.assertEqual(errors[0].message, u'Required input is missing.')

    def testProxyRole(self):
        """ Test seeing how setting proxy role affects unauthorized
        Exception
        """
        self.createScript()

        actions = get_actions(self.ff1)
        adapter = actions['adapter']

        # 4. Set script data
        adapter.ScriptBody = proxied_script

        req = SecureFakeRequest(test_field='123')
        self.assertRaises(Unauthorized, adapter.onSuccess, {}, req)



# -*- coding: utf-8 -*-
from Products.PageTemplates.Expressions import getEngine
from zope.tales import tales

import zope.schema.interfaces


class InvalidTALESError(zope.schema.ValidationError):
    __doc__ = u'Please enter a valid TALES expression.'


def isTALES(value):
    if value.strip():
        try:
            getEngine().compile(value)
        except tales.CompilerError:
            raise InvalidTALESError
    return True



# -*- coding: utf-8 -*-
from collective.easyform import easyformMessageFactory as _
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.registry.interfaces import IRegistry
from plone.z3cform import layout
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.component import getUtility
from zope.interface import Interface


def getContent(self):
        return getUtility(IRegistry).forInterface(
            self.schema,
            prefix=self.schema_prefix)


class IEasyFormControlPanel(Interface):

    migrate_all_forms = schema.Bool(
        title=u'migrate all the forms to dexterity',
        description=u'This will migrate all the forms already present '
                    u'in the site from archetype to dexterity',
        required=False,
        default=False,
    )

    allowedFields = schema.List(
        title=_(u"Allowed Fields"),
        description=_(u"This Fields are available for your forms."),
        value_type=schema.Choice(
            description=_(
                u"help_registry_items",
                default=u"Select the registry items you desire to modify"
            ),
            required=False,
            vocabulary='easyform.SchemaEditorFields',
        ),
        default=[],
    )


class EasyFormControlPanelForm(RegistryEditForm):
    schema = IEasyFormControlPanel
    schema_prefix = 'easyform'
    label = u'easyform Settings'

    def updateFields(self):
        super(EasyFormControlPanelForm, self).updateFields()
        self.fields['allowedFields'].widgetFactory = \
            CheckBoxFieldWidget


EasyFormControlPanelView = layout.wrap_form(
    EasyFormControlPanelForm, ControlPanelFormWrapper)



# -*- coding: utf-8 -*-
#
# Test export and import of easyforms via GenericSetup
#

from cgi import FieldStorage
from collective.easyform.tests import base
from plone import api
from Products.GenericSetup.tests.common import DummyExportContext
from Products.GenericSetup.tests.common import TarballTester
from StringIO import StringIO
from tarfile import TarFile
from zope.component import getMultiAdapter
from ZPublisher.HTTPRequest import FileUpload

import os
import re


class ExportImportTester(base.EasyFormTestCase, TarballTester):

    """Base class for integration test suite for export/import """

    def afterSetUp(self):
        portal_setup = api.portal.get_tool(name='portal_setup')
        portal_setup.runAllImportStepsFromProfile(
            'profile-collective.easyform:testing')

    def _makeForm(self):
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')

    def _prepareFormTarball(self):
        """we could use our @@export-easyform view,
           but we want a more atomic test, so we make
           a tarfile for our test.  the approach to making
           a tarfile is a bit strange, but does the job
        """
        in_fname = 'test_form_1_easyform.tar.gz'
        test_dir = os.path.dirname(__file__)

        def _add_form_structure_to_archive(archive):
            form_relative_path = os.path.join(
                'profiles', 'testing', 'structure',
                'Members', 'test_user_1_', 'test_form_1_easyform')
            abs_path = os.path.join(test_dir, form_relative_path)

            # add structure folder
            os.chdir(os.path.join(test_dir, 'profiles', 'testing'))
            archive.add('structure', recursive=False)

            for f in os.listdir(abs_path):
                os.chdir(abs_path)  # add form data w/o full directory tree
                archive.add(f, arcname=os.path.join('structure', f))

        # Capture the current working directory for later when we need to
        # clean up the environment.
        working_directory = os.path.abspath(os.curdir)

        # make me a tarfile in the current dir
        os.chdir(test_dir)
        archive = TarFile.open(name=in_fname, mode='w:gz')
        _add_form_structure_to_archive(archive)
        archive.close()

        # Change back to the working directory in case something tries to
        # write files (e.g. collective.xmltestreport).
        os.chdir(working_directory)

        # get it and upload
        in_file = open(os.path.join(test_dir, in_fname))
        env = {'REQUEST_METHOD': 'PUT'}
        headers = {'content-type': 'text/html',
                   'content-length': len(in_file.read()),
                   'content-disposition': 'attachment; filename={0}'.format(in_file.name)}  # noqa
        in_file.seek(0)
        fs = FieldStorage(fp=in_file, environ=env, headers=headers)
        return FileUpload(fs)

    def _extractFieldValue(self, setval):
        """differentiate datastructure for TALESField instances vs.
           standard ATField instances
        """
        if hasattr(setval, 'raw'):
            setval = setval.raw

        return setval

    def _verifyProfileFormSettings(self, form_ctx):
        form_values = {
            'title': 'My OOTB Form',
            # 'isDiscussable':False,
            'description': 'The description for our OOTB form',
            # XXX andrewb enable when form props are supported
            # 'submitLabel':'Hit Me',
        }

        for k, v in form_values.items():
            self.assertEqual(
                v,
                self._extractFieldValue(form_ctx[k]),
                "Expected '{0}' for field {1}, Got '{2}'".format(v, k, form_ctx[k])  # noqa
            )

    def _verifyFormStockFields(self, form_ctx, purge):
        """ helper method to verify adherence to profile-based
            form folder in tests/profiles/testing directory
        """
        form_field_ids = form_ctx.objectIds()
        check_fields = ('comments', 'replyto', 'topic', 'mailer', 'thank-you')
        for form_field in check_fields:
            if purge:
                self.assertNotIn(
                    form_field,
                    form_field_ids,
                    '{0} unexpectedly found in {1}'.format(form_field, form_ctx.getId())  # noqa
                )
                continue

            self.assertIn(
                form_field,
                form_field_ids,
                '{0} not found in {1}'.format(form_field, form_ctx.getId())
            )

    def _verifyProfileForm(self, form_ctx, form_fields=None):
        """ helper method to verify adherence to profile-based
            form folder in tests/profiles/testing directory
        """
        form_id_prefix = 'test_form_1_'

        if not form_fields:
            form_fields = [
                {
                    'id': '{0}replyto'.format(form_id_prefix),
                    'title': 'Test Form Your E-Mail Address',
                    'required': True,
                    'fgTDefault': 'here/memberEmail', },
                {
                    'id': '{0}hidden'.format(form_id_prefix),
                    'title': 'This is a sample hidden field',
                    'required': False,
                    'hidden': True, },
                {
                    'id': '{0}fieldset-folder'.format(form_id_prefix),
                    'title': 'Fields grouped in a fieldset',
                    'useLegend': False,
                    'subfields': [{
                        'id': '{0}hidden_fieldset'.format(form_id_prefix),
                        'title': 'This is a sample hidden field fieldset',
                        'required': True,
                        'hidden': True,
                    }, ], },
                {
                    'id': '{0}topic'.format(form_id_prefix),
                    'title': 'Test Form Subject',
                    'required': True, },
                {
                    'id': '{0}comments'.format(form_id_prefix),
                    'fgDefault': 'string:Test Comment', },
            ]
        # get our forms children to ensure proper config
        for form_field in form_fields:
            self.assertTrue(form_field['id'] in form_ctx.objectIds())
            sub_form_item = form_ctx[form_field['id']]
            # make sure all the standard callables are set
            for k, v in form_field.items():
                if k == 'subfields':
                    self._verifyProfileForm(sub_form_item, v)
                else:
                    self.assertEqual(
                        v,
                        self._extractFieldValue(sub_form_item[k]),
                        "Expected '{0}' for field {1}, Got '{2}'".format(v, k, sub_form_item[k])  # noqa
                    )


class TestFormExport(ExportImportTester):

    """Export Form Test Suite"""

    file_tmpl = 'structure/{0}'
    title_output_tmpl = 'title: {0}'

    def _getExporter(self):
        from Products.CMFCore.exportimport.content import exportSiteStructure
        return exportSiteStructure

    def test_stock_form_contextual_export(self):
        """Upon adding a form folder, we're given a fully functional
           example form that provides several fields, a mailer, and
           a thanks page.  Let's make sure a contextual export includes what
           we're expecting.
        """
        self._makeForm()
        context = DummyExportContext(self.ff1)
        exporter = self._getExporter()
        exporter(context)

        # shove everything into a dictionary for easy inspection
        form_export_data = {}
        for filename, text, content_type in context._wrote:
            form_export_data[filename] = text

        # make sure our field and adapters are objects
        for id, object in self.ff1.objectItems():
            self.assertTrue((self.file_tmpl.format(id)) in form_export_data,
                            'No export representation of {0}'.format(id))
            self.assertTrue(self.title_output_tmpl.format(object.Title()) in
                            form_export_data[self.file_tmpl.format(id)])

        # we should have .properties, .objects, and per subject
        self.assertEqual(len(context._wrote), 2 + len(self.ff1.objectIds()))

    def test_form_properties_contextual_export(self):
        """In order to accurately export the schema values for our
           EasyForm, we forego the otherwise adequate
           StructureFolderWalkingAdapter's ConfigParser format export
           of .properties for favor of an RFC 822  (email) style string
           export.  Confirm that is so here.
        """
        self._makeForm()
        self.ff1.submitLabel = 'Hit Me'
        context = DummyExportContext(self.ff1)
        exporter = self._getExporter()
        exporter(context)

        # shove everything into a dictionary for easy inspection
        form_export_data = {}
        for filename, text, content_type in context._wrote:
            form_export_data[filename] = text

        ff1_props = form_export_data['structure/.data']

        lab_pat = re.compile(r'submitLabel.*?Hit Me')
        self.assertTrue(lab_pat.search(ff1_props))

    def ttest_fieldset_properties_contextual_export(self):
        """In order to accurately export the schema values for our
           FieldsetFolder, we forego the otherwise adequate
           StructureFolderWalkingAdapter's ConfigParser format export
           of .properties for favor of an RFC 822 (email) style string
           export.  Confirm that is so here.
        """
        self._makeForm()
        self.ff1.invokeFactory('FieldsetFolder', 'fsf1')
        self.ff1.fsf1.setTitle('EasyForm1 FieldsetFolder1')
        self.ff1.fsf1.setUseLegend(False)  # set a non-default value
        context = DummyExportContext(self.ff1)
        exporter = self._getExporter()
        exporter(context)

        # shove everything into a dictionary for easy inspection
        form_export_data = {}
        for filename, text, content_type in context._wrote:
            form_export_data[filename] = text

        fsf1_props = form_export_data['structure/fsf1/.properties']

        self.assertTrue('EasyForm1 FieldsetFolder1' in fsf1_props)
        leg_pat = re.compile(r'useLegend.*?False')
        self.assertTrue(leg_pat.search(fsf1_props))

    def test_stock_form_view_export(self):
        """We provide a browser view that can be used to
           export a given form as the root context as well.
           XXX - Andrew B remember this is a rather tempoary representation
           of what's returned.
        """
        toc_list = ['structure/.objects', 'structure/.data']
        self._makeForm()
        form_folder_export = getMultiAdapter(
            (self.folder.ff1, self.app.REQUEST),
            name='export-easyform')
        fileish = StringIO(form_folder_export())
        try:
            self._verifyTarballContents(fileish, toc_list)
        except AssertionError:
            toc_list.append('structure')
            self._verifyTarballContents(fileish, toc_list)


class TestFormImport(ExportImportTester):

    """Import Form Test Suite"""

    def _getImporter(self):
        from Products.CMFCore.exportimport.content import importSiteStructure
        return importSiteStructure

    def ttest_form_values_from_gs_import(self):
        """We provide a custom IFilesystemImporter so that
           all the schema fields from a configured EasyForm
           land in the imported form.
        """
        self.assertTrue('test_form_1_easyform' in self.folder.objectIds())
        self._verifyProfileFormSettings(self.folder['test_form_1_easyform'])

    def ttest_profile_from_gs_import(self):
        """We create a profile (see: profiles/testing/structure)
           which adds a EasyForm called 'test_form_1_' in via our
           import handler. The form uses the standard ids proceeded by
           test_form_1_.  We test for the existence of and accurate
           configuration of these subfields below.
        """
        # did our gs form land into the test user's folder
        self.assertTrue('test_form_1_easyform' in self.folder.objectIds())
        self._verifyProfileForm(self.folder['test_form_1_easyform'])

    def test_formlib_form_import(self):
        """Interacting with our formlib form we should be able
           to successfully upload an exported PFG form and have it
           correctly configured.
        """
        self._makeForm()

        fields = self.ff1.fields_model
        actions = self.ff1.actions_model

        form_folder_export = getMultiAdapter(
            (self.ff1, self.app.REQUEST),
            name='export-easyform')
        in_file = StringIO(form_folder_export())
        env = {'REQUEST_METHOD': 'PUT'}
        headers = {'content-type': 'text/html',
                   'content-length': len(in_file.read()),
                   'content-disposition': 'attachment; filename=ff1.tar.gz'}
        in_file.seek(0)
        fs = FieldStorage(fp=in_file, environ=env, headers=headers)

        # setup a reasonable request
        request = self.app.REQUEST
        request.form = {
            'form.widgets.upload': FileUpload(fs),
            'form.buttons.import': 'import'}
        request.RESPONSE = request.response

        # get the form object
        import_form = getMultiAdapter(
            (self.ff1, request), name='import-easyform')

        # call update (aka submit) on the form, see TestRequest above
        import_form.update()
        self.assertEqual(self.ff1.fields_model, fields)
        self.assertEqual(self.ff1.actions_model, actions)

    def ttest_formlib_form_with_purge_import(self):
        self._makeForm()

        # submit the form requesting purge of contained fields
        request = self.app.REQUEST
        request.form = {
            'form.widgets.upload': self._prepareFormTarball(),
            'form.buttons.import': 'import'}
        request.RESPONSE = self.app.REQUEST.response

        # get the form object
        import_form = getMultiAdapter(
            (self.ff1, request), name='import-easyform')

        # call update (aka submit) on the form, see TestRequest above
        import_form.update()
        self._verifyProfileForm(self.ff1)
        self._verifyFormStockFields(self.ff1, purge=True)



# -*- coding: utf-8 -*-
#
# Test EasyForm event-handler functionality
#

from collective.easyform.api import get_actions
from collective.easyform.api import get_schema
from collective.easyform.tests import base


class TestAdapterPaste(base.EasyFormTestCase):

    """Ensure content types can be created and edited"""

    adapterTypes = (
        'FormSaveDataAdapter',
        'FormMailerAdapter',
        'FormCustomScriptAdapter',
    )

    def afterSetUp(self):
        super(TestAdapterPaste, self).afterSetUp()
        self.folder.invokeFactory('EasyForm', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')

    def testActiveAdaptersNotDuplicatedOnFormCopy(self):
        # self.loginAsPortalOwner()
        copy = self.folder.manage_copyObjects('ff1')
        new_id = self.folder.manage_pasteObjects(copy)[0]['new_id']
        ff2 = getattr(self.folder, new_id)
        active_adapters = tuple(get_actions(ff2))
        self.assertEqual(active_adapters, ('mailer',))
        active_fields = tuple(get_schema(ff2))
        self.assertEqual(active_fields, ('replyto', 'topic', 'comments'))
